/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/////////////////////////////////////////////////////////////////////
// ax25.h
//
//                           -------------------
//  begin                : Wed Feb  5 11:47:20 CST 2003
//  copyright            : (C) 2003 by John Bennett
//  email                : n4xi@arrl.net
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
////////////////////////////////////////////////////////////////////////////

#ifndef _AX25_H_
#define _AX25_H_

#include <sys/socket.h>

#ifndef BOOL
    typedef int BOOL;
#endif


#ifndef TRUE
    #define TRUE 1
    #define FALSE 0
#endif

#define _RAWFRAME

#define AX25_FRAME_MAX_SIZE  512

#define AX25_FLAG_SIZE  1
#define ADDR_SIZE       7
#define ADDR_FIELD_SIZE ((10*ADDR_SIZE)+1)
#define CF_SIZE         1
#define PID_SIZE        1
#define MAX_DIGI        8
#define INF_MAX_SIZE    256

#define AX25_CALLSSID_SIZE  10

#define AX25_SPACE  0x40    // space left-shifted one bit
#define AX25_FLAG   0x7e    // frame delimiter

#define AX25_DIGI_SIZE ( ( (AX25_CALLSSID_SIZE)*8 ) + 1 )

#define INDEX_RESET   1
#define INDEX_NEXT    2
#define INDEX_PREV    3


// Control Field

#define NO_FRAME    0
#define I_FRAME     1
#define S_FRM_RR    2
#define S_FRM_RNR   3
#define S_FRM_REJ   4
#define U_FRM_UI    5
#define U_FRM_FRMR  6
#define U_FRM_UA    7
#define U_FRM_DM    8
#define U_FRM_DISC  9
#define U_FRM_SABM  10

#define NO_SFRAME   98
#define NO_UFRAME   99

// U-frame control field defs

#define UI      0x03
#define FRMR    0x87
#define UA      0x63
#define DM      0x0f
#define DISC    0x43
#define SABM    0x2f

// I-frame control field defs


// S-frame control field defs

#define RR      0x01
#define RNR     0x05
#define REJ     0x09

// PID defs

#define IP_DATAGRAM_L3  0xcc
#define ARP_L3          0xcd
#define NO_L3           0xf0
#define PID_ESC         0xff

#define AXPORT_SIZE     16



struct ax25call
{
    char szCall[8];
    int nSsid;
};


class Ax25
{
    public:
        Ax25();
        ~Ax25();

        BOOL setup( char * pMyCall, char * szPort, BOOL bRcvAll );
        void put( char * pCall, char * pDigi, char * pInf, int nInfLen );
        BOOL get( void );
        int data( char * pData, int nMaxLen );
        void ToCall( char * pCall );
        int ToSsid( void );
        void FmCall( char * pCall );
        int FmSsid( void );
        int RxPid( void );
        int FrameType( void );
        int InfLength( void );
        int UiFrameCount( void );

        int DigiCount( void );
        BOOL DigiIndex( int nAction );
        void DigiCall( char * pCall );

        #ifdef _RAWFRAME
            char * RawFrame( void );
            unsigned RawFrameLength( void );
        #endif

    private:
        int SetAddr( char * pDest, char * pDigi );
        void EncodeCall( char * pCall );
        void DecodeCall(  char * pCall, struct ax25call * pResult  );
        BOOL DecodeFrameType( char * pByte );

        char m_MyCall[AX25_CALLSSID_SIZE];
        char m_AddrField[ADDR_FIELD_SIZE];

        unsigned char m_TxPid;

        unsigned char m_TxCf;

        struct ax25call m_ToCall;
        struct ax25call m_FmCall;
        struct ax25call m_Digi[8];

        unsigned char m_RxPid;  // protocol ID field
        int m_nRxCf;            // control field type
        int m_nDigiCnt;
        int m_nDigiIndex;

        int m_nUiFrameCount;

        int m_nRxDataLength;
        int m_nFrameLength;

        char * m_pRxData;

        char m_RxKissFrame[AX25_FRAME_MAX_SIZE];
        char m_TxKissFrame[AX25_FRAME_MAX_SIZE];

        char m_szPort[AXPORT_SIZE];

        int m_socket;
        struct sockaddr m_sa;
};


#endif
