/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// logmsg.cpp  -  post log message facility
//

#define _LOGMSG_CPP_

#include <string.h>
#include <stdio.h>
#include <time.h>

#include "logmsg.h"
#include "textfile.h"

// These strings must appear in the same order that they appear in <logmsg.h>!
// Note that indices in comments are for human reference only to easily track
// messages in a given area. The index itself starts at zero and increments
// by one. e.g., the reference of '100' will not have an index of that value.
// It will be one greater than the last entry in the previous group.

static const char * szLogMsg[] =
{
    // general

    /*000*/     "Debug Message",

    /*001*/     "Error starting daemon - terminating",
    /*002*/     "Stopped",

    /*003*/     "<wxncommon.conf> error",

    /*004*/     "Attempting to start...",
    /*005*/     "Startup succeeded",
    /*006*/     "Startup failed",

    /*007*/     "Initializing...",
    /*008*/     "Initialization failed",
    /*009*/     "Initialization complete",

    /*010*/     "Thread start fail",


    // wxnhostd

    /*100*/     "<wxnhostd.conf> error",
    /*101*/     "Could not load/create local record",
    /*102*/     "Could not open wx stn serial port",
    /*103*/     "Error opening one-wire port, sensors disabled",
    /*104*/     "Wx stn read fail",
    /*105*/     "Wx stn data not available",
    /*106*/     "OW inside humidity read failed",
    /*107*/     "OW outside humidity read failed",
    /*108*/     "Wx stn not defined",
    /*109*/     "Initial OW barometer read fail - sensor disabled",
    /*110*/     "Initial OW outdoor humidity read fail - sensor disabled",
    /*111*/     "ReadWxStn() buffer overrun",
    /*112*/     "ReadWxStn() timeout",
    /*113*/     "OwReadCounter() fail",
    /*114*/     "OwReadTemp() fail",
    /*115*/     "OwReadBarom() fail",
    /*116*/     "ResetLink() fail",
    /*117*/     "ReadWxStnBytes() timeout",
    /*118*/     "SendId5Cmd() fail",
    /*119*/     "GetId5Data() error",
    /*120*/     "Id5ProcessData() could not restart auto-send",
    /*121*/     "OwReadHumid() fail",
    /*122*/     "CRC fail",
    /*123*/     "Invalid parameter",
    /*124*/     "DavPProcessData() - no blocks to process",
    /*125*/     "DS2423 counter not found or cannot be determined",
    /*126*/     "DS2405 PIO not found or cannot be determined",
    /*127*/     "DS18S20 serial number must be specified in <wxnhostd.conf>",
    /*128*/     "DS2438 serial number must be specified in <wxnhostd.conf>",
    /*129*/     "Illeagal baud rate specified",
    /*130*/     "GetBlock() failed",
    /*131*/     "WxPic port failed to open",
    /*132*/     "socket failure",
    /*133*/     "socket receive failure",
    /*134*/     "Alarm trip points read from database",
    /*135*/     "Alert email failure",
    /*136*/     "Outside temp read error(s) last hour",
    /*137*/     "Inside temp read error(s) last hour",
    /*138*/     "Wind speed read error(s) last hour",
    /*139*/     "Wind direction read error(s) last hour",
    /*140*/     "Barometer read error(s) last hour",
    /*141*/     "Precip read error(s) last hour",
    /*142*/     "Outside humidity read error(s) last hour",
    /*143*/     "Inside humidity read error(s) last hour",
    /*144*/     "Wx Stn CRC fail last hour",

    // wxnaprd

    /*200*/     "<wxnaprd.conf> error",
    /*201*/     "QRZ interface init failed. If used, is CDROM mounted?",
    /*202*/     "Unable to read local node record for beacon",
    /*203*/     "AprsReceive() - Invalid data for",
    /*204*/     "AprsReceive(): ReadZoneInfo() error",
    /*205*/     "AprsReceive(): WriteBase() error",
    /*206*/     "AprsReceive(): WriteCalc() error",
    /*207*/     "FIPS lookup failed",
    /*208*/     "Aprs.rxData() failed - reason unknown",
    /*209*/     "Could not update record in <nodes> table",
    /*210*/     "GetGeoData() failed",
    /*211*/     "deny list re-init",
    /*212*/     "String overflow in aprs::WxnTx()",
    /*213*/     "Ax25::setup() @ socket()",
    /*214*/     "Ax25::put() @ sendto()",
    /*215*/     "Ax25::get() @ recvfrom()",
    /*216*/     "Failed to create record",
    /*217*/     "Qrz::SetIndex() failed",
    /*218*/     "Could not find/open in '/etc/wxn/' to initialize list for",
    /*219*/     "Access denied:",

    // wxnqueryd

    /*300*/     "<wxnqueryd.conf> error",
    /*301*/     "Ax25::setup() @ socket()",
    /*302*/     "Ax25::put() @ sendto()",
    /*303*/     "Ax25::get() @ recvfrom()",
    /*304*/     "Failed to create record in 'query' table for",

    // wxnsamed

    /*400*/     "Could not load/read 'wxnsamed.conf'",
    /*401*/     "Unable to open serial port for SAME receiver",
    /*402*/

};

logmsg::logmsg()
{
    *m_sLogFile = 0;
    m_bVerbose = FALSE;
}

logmsg::~logmsg() {}


void logmsg::init( char *sLogFile, BOOL bVerbose )
{
    strcpy( m_sLogFile, sLogFile );
    m_bVerbose = bVerbose;
}

char * logmsg::dir( void )
{
    return m_sLogFile;
}


void logmsg::PostMsg( int nMsg, const char * szMsg )
{
    textfile file;
    time_t tTime;
    char *pS;
    char szDTG[48];
    BOOL bOpen;

    if( *m_sLogFile )
        bOpen = file.open( m_sLogFile, FILE_APPEND );
    else
        bOpen = FALSE;

    tTime = time( NULL );
    strcpy( szDTG, ctime( &tTime ) );
    pS = strchr( szDTG, '\n' );
    if( pS )
        *pS = 0;

    if( szMsg )
    {
        if( m_bVerbose )
            fprintf( stderr, "%s: %s\n", szLogMsg[nMsg], szMsg );
        if( bOpen )
            fprintf( file.handle(), "%s - %s: %s\n", szDTG, szLogMsg[nMsg], szMsg );
    }
    else
    {
        if( m_bVerbose )
            fprintf( stderr, "%s\n", szLogMsg[nMsg] );
        if( bOpen )
            fprintf( file.handle(), "%s - %s\n", szDTG, szLogMsg[nMsg] );
    }
}

