/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// tz.cpp - timezone class handles dates/times for given locale
//

#define _TZ_CPP_

//#define _BSD_SOURCE

#include <time.h>
#include <sys/timeb.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "tz.h"


// global instance
tz g_Time;


// class implementation

tz::tz(){}

tz::~tz(){}

void tz::Init( void )
{
    struct tm * pTm;

    m_tServerStart = time( NULL );

    m_Minute = 0;
    m_LastMinute = 0;

    // Since there is no sane way to set yesterday's values, set to today's date.
    // This will be set correctly at midnight local time. Should not be used until
    // then. These values are used for debugging purposes.

    pTm = localtime( &m_tServerStart );
    m_Yesterday.day = pTm->tm_mday;
    m_Yesterday.month = pTm->tm_mon + 1;
    m_Yesterday.year = pTm->tm_year + 1900;

}

void tz::Set( void )
{
    time_t tNow;
    struct tm * pTm;

    // get current time
    tNow = time( NULL );

    // parse out local dtg info
    pTm = localtime( &tNow );
    if( (pTm->tm_hour | pTm->tm_min) == 0 )
    {
        m_Yesterday.day = m_Day;
        m_Yesterday.month = m_Month;
        m_Yesterday.year = m_Year;
        m_bMidnight = TRUE;
    }
    else
        m_bMidnight = FALSE;

    m_tUtc = tNow;
    m_tUtcOffset = pTm->tm_gmtoff;
    m_tQth = tNow + m_tUtcOffset;

    m_Month = pTm->tm_mon + 1;
    m_Day = pTm->tm_mday;
    m_Year = pTm->tm_year + 1900;
    m_Hour = pTm->tm_hour;
    m_Second = pTm->tm_sec;
    m_YearDay = pTm->tm_yday + 1;
    m_DayMinute = pTm->tm_hour*60 + pTm->tm_min;

    m_bDstNow = pTm->tm_isdst;

    if( m_LastMinute != pTm->tm_min )
        m_LastMinute = m_Minute;
    m_Minute = pTm->tm_min;

}


time_t tz::Qth( void )
{
    return m_tQth;
}

time_t tz::Utc( void )
{
    return m_tUtc;
}

int tz::Year( void )
{
    return m_Year;
}

int tz::Month( void )
{
    return m_Month;
}

int tz::Day( void )
{
    return m_Day;
}

int tz::Hour( void )
{
    return m_Hour;
}

int tz::Minute( void )
{
    return m_Minute;
}

int tz::Second( void )
{
    return m_Second;
}

int tz::YearDay( void )
{
    return m_YearDay;
}

int tz::DayMinute( void )
{
    return m_DayMinute;
}

int tz::PrevDay( void )
{
    return m_Yesterday.day;
}

int tz::PrevMonth( void )
{
    return m_Yesterday.month;
}

int tz::PrevYear( void )
{
    return m_Yesterday.year;
}

BOOL tz::NewDay( void )
{
    return m_bMidnight;
}

BOOL tz::NewMonth( void )
{
    if( m_Month != m_Yesterday.month )
        return TRUE;
    else
        return FALSE;
}


BOOL tz::NewYear( void )
{
    if( m_Year != m_Yesterday.year )
        return TRUE;
    else
        return FALSE;
}


BOOL tz::DstNow( void )
{
    return m_bDstNow;
}


time_t tz::Runtime( void )
{
    return m_tQth - m_tServerStart;
}

char * tz::MakeLocalTimestamp( char * pS, time_t tDtg )
{
    struct tm * dtg;

    dtg = gmtime( &tDtg );

    sprintf( pS, "%i-%.2i-%.2i %.2i:%.2i", dtg->tm_year+1900, dtg->tm_mon+1, dtg->tm_mday, dtg->tm_hour, dtg->tm_min );

    return pS;
}


char * tz::MakeNodeTimestamp( char * pS, int nZone, BOOL bDst )
{
    struct tm * dtg;
    time_t tLocal;

    tLocal = m_tUtc - (time_t) (nZone*HOUR_SEC);
    if( bDst && m_bDstNow )
        tLocal += HOUR_SEC;

    dtg = gmtime( &tLocal );

    sprintf( pS, "%i-%.2i-%.2i %.2i:%.2i", dtg->tm_year+1900, dtg->tm_mon+1, dtg->tm_mday, dtg->tm_hour, dtg->tm_min );

    return pS;
}


char * tz::CurrentZuluTimestamp( char * pS )
{
    struct tm * dtg;

    dtg = gmtime( &m_tUtc );

    sprintf( pS, "%i-%.2i-%.2i %.2i:%.2i", dtg->tm_year+1900, dtg->tm_mon+1, dtg->tm_mday, dtg->tm_hour, dtg->tm_min );

    return pS;
}


char * tz::CurrentLocalTimestamp( char * pS )
{
    struct tm * dtg;

    dtg = gmtime( &m_tQth );

    sprintf( pS, "%i-%.2i-%.2i %.2i:%.2i", dtg->tm_year+1900, dtg->tm_mon+1, dtg->tm_mday, dtg->tm_hour, dtg->tm_min );

    return pS;
}


time_t tz::dtg( char * pString )
{
    struct tm dtg;
    time_t tMake;
    char szTemp[16];

    bzero( &dtg, sizeof( struct tm ) );

    memcpy( szTemp, pString, 4 );
    szTemp[4] = 0;
    dtg.tm_year = atoi( szTemp ) - 1900;

    memcpy( szTemp, pString+5, 2 );
    szTemp[2] = 0;
    dtg.tm_mon = atoi( szTemp ) - 1;

    memcpy( szTemp, pString+8, 2 );
    szTemp[2] = 0;
    dtg.tm_mday = atoi( szTemp );

    memcpy( szTemp, pString+11, 2 );
    szTemp[2] = 0;
    dtg.tm_hour = atoi( szTemp );

    memcpy( szTemp, pString+14, 2 );
    szTemp[2] = 0;
    dtg.tm_min = atoi( szTemp );

    // set to unknown
    dtg.tm_isdst = -1;

    tMake = mktime( &dtg ) + m_tUtcOffset;

    return tMake;
}


void tz::Date( char * pString )
{
    sprintf( pString, "%i-%2i-%2i", m_Year, m_Month, m_Day );
}

//////////////////////////////////////////////////////////////////////////////
// MakeNodeDtg - needs to be fixed so that time is corrected for node's time
// zone.
//

time_t tz::MakeNodeDtg( char * psTime )
{
    char szTemp[32];

    sprintf( szTemp, "%i-%2i-%2i %s", m_Year, m_Month, m_Day, psTime );

    return dtg( szTemp );
}


//////////////////////////////////////////////////////////////////////////////
// MakeNodeHrMin - returns the hour and minute in 'hh:mm' from tTime
//

void tz::MakeNodeHrMin( time_t tTime, char * psTime )
{
    struct tm * dtg;

    dtg = gmtime( &tTime );

    sprintf( psTime, "%.2i:%.2i", dtg->tm_hour, dtg->tm_min );
}


//////////////////////////////////////////////////////////////////////////////
// MakeRiseSet -
//

void tz::MakeRiseSet( time_t tTime, char * psTime )
{
    int nHour;
    int nMin;

    nHour = (int) tTime / 60;
    nMin = (int) tTime - (nHour * 60);   // prev int div dropped min
    sprintf( psTime, "%.4i-%.2i-%.2i %.2i:%.2i",
        Year(), Month(), Day(), nHour, nMin );
}


//////////////////////////////////////////////////////////////////////////////
// MakeTime - returns 'hh:mm' form in seconds
//

time_t tz::MakeTime( char * szTime )
{
    char szTemp[16];
    int nVal;

    szTemp[2] = 0;

    memcpy( szTemp, szTime, 2 );
    nVal = atoi( szTemp ) * HOUR_SEC;

    memcpy( szTemp, szTime+3, 2 );
    nVal += atoi( szTemp ) * MINUTE_SEC;

    return (time_t) nVal;
}

