/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// tz.h -  provides time/date related functions for a given locale
//

#ifndef _TZ_H_
#define _TZ_H_

#include <time.h>

#ifndef BOOL
    typedef int BOOL;
#endif

#ifndef TRUE
    #define TRUE    1
    #define FALSE   0
#endif

// misc constants

#define HOST_INTERVAL           5   // minutes between wx stn reads
#define WXPIC_TIMER_INTERVAL    5   // seconds

// used for rates

#define HOUR                    60
#define HOUR_INTERVAL_COUNT     (HOUR/HOST_INTERVAL)


#define MINUTE_SEC      (time_t) 60
#define HOUR_SEC        (time_t) 3600           // 60*60
#define DAY_SEC         (time_t) 86400          // 60*60*24

struct yesterday
{
    int day;
    int month;
    int year;
};


class tz {

public:
    tz();
    ~tz();

    void Init( void );

    void Set( void );

    time_t Qth( void );
    time_t Utc( void );
    int Year( void );
    int Month( void );
    int Day( void );
    int Hour( void );
    int Minute( void );
    int Second( void );
    int YearDay( void );
    int DayMinute( void );

    int PrevDay( void );
    int PrevMonth( void );
    int PrevYear( void );

    BOOL DstNow( void );

    BOOL NewDay( void );
    BOOL NewMonth( void );
    BOOL NewYear( void );

    time_t Runtime( void );
    char * MakeLocalTimestamp( char * pS, time_t tDtg );
    char * MakeNodeTimestamp( char * pS, int nZone, BOOL bDst );
    char * CurrentZuluTimestamp( char * pS );
    char * CurrentLocalTimestamp( char * pS );
    time_t dtg( char * pString );
    void Date( char * pString );
    time_t MakeNodeDtg( char * pS );
    void MakeNodeHrMin( time_t tTime, char * psTime );
    void MakeRiseSet( time_t tTime, char * psTime );
    time_t MakeTime( char * szTime );

    yesterday m_Yesterday;

private:

    time_t m_tQth;
    time_t m_tUtc;

    time_t m_tUtcOffset;

    int m_Year;
    int m_Month;
    int m_Day;
    int m_Hour;
    int m_Minute;
    int m_Second;
    int m_LastMinute;
    int m_YearDay;
    int m_DayMinute;

    BOOL m_bDstNow;
    BOOL m_bMidnight;

    time_t m_tServerStart;      // must be based on local (QTH) time

};


#ifndef _TZ_CPP_
    extern tz g_Time;
#endif


#endif  // _TZ_H_
