/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// aprs.h - class handler for native APRS functions
//

#ifndef _APRS_H_
#define _APRS_H_

// libwxn
#include <ax25.h>
#include <datastruct.h>

#include "dupe.h"


#define APRS_SOFT_TYPE 'n'
#define WXN_USER_ID    'W'

#define COORDINATES_SIZE    20
#define STN_SIZE            5

// User defined data types for APRS

#define NO_TYPE 0
#define TYPE_A  101
#define TYPE_B  102
#define TYPE_C  103
#define TYPE_D  104
#define TYPE_E  105

// return values for status

#define NO_DATA         0x00
#define WX_DATA         0x01
#define POSITION        0x02
#define WXN_DATA        0x04

#define IGATE           0x80
#define IGATE_MASK      0x7f

// data type identifier

#define INVALID_TYPE    0
#define POSITIONLESS    1
#define TIME_UPOS_WX    2
#define TIME_UPOS       3
#define TIME_CPOS_WX    4
#define TIME_CPOS       5
#define UPOS_WX         6
#define UPOS            7
#define CPOS_WX         8
#define CPOS            9
#define USR_WXN         10

#define WIND_DIR_LEN    3
#define WIND_SPD_LEN    3
#define GUST_LEN        3


#define TEMP_LEN        3
#define RAIN_HR_LEN     3
#define RAIN_24_LEN     3
#define RAIN_DAY_LEN    3
#define HUMID_LEN       2
#define BAROM_LEN       5

#define CPOS_LENGTH     9
#define UPOS_LENGTH     18

// the following two must include leading underscore symbol in count
#define CWND_LENGTH     4       // _cst
#define UWND_LENGTH     8       // _ccc/sss

#define TIMESTAMP_LEN   7
#define NOPOS_STAMP_LEN 8

#define SYMBOL_LEN      1

#define START_OFFSET    1


// software type used by an APRS node

#define WXN         1
#define APRSSOFT    2
#define APRSDOS     3
#define MACAPRS     4
#define POCKETAPRS  5
#define APRSSA      6
#define WINAPRS     7
#define XAPRS       8
#define UNIXAPRS    9
#define APRSPOINT   10


struct position
{
    double dLat;
    double dLng;
};

struct aprswind
{
    int nAzimuth;
    int nSpeed;
};


class Aprs : public Ax25
{
    public:

        Aprs();
        ~Aprs();

        BOOL setup( char * pFmCall, char * pToCall, char * pDigis, char * pPort, char * pText, BOOL bRcvLocal );
        void extra( char * pText );
        char * extra( void );
        void SetStationType( int nType );
        void SetCoordinates( double dLatitude, double dLongitude );

        void RxFmCall( char * pS );
        void RxToCall( char * pS );

        void digis( char * pDigis );
        char * digis( void );

        void beacon( noderx * pBlock );
        void BeaconExtra( void );
        void WxnTx( int nType, noderx * pBlock );

        int rxStatus( void );
        BOOL rxData( noderx * pBlock, time_t tZulu  );
        BOOL rxPos( position * pPos );

        BOOL Position( void );
        double rxLatitude( void );
        double rxLongitude( void );

        int DecodeType( void );

        int IGateCount( void );
        BOOL IGateIndex( int nAction );
        void IGateDigi( char * pCall );

    private:

        int DecodePosition( int nOffset );
        time_t DecodeTime( time_t tTime );
        int DecodeWind( nodedata * pData, int nOffset );
        void WeatherData( noderx * pBlock, int nOffset, BOOL bWind );
        int WxnDecode( noderx * pBlock );
        void BeaconWxn( char * pData );

        Dupe m_dupe;

        char m_sInf[INF_MAX_SIZE];
        char m_TxToCall[ADDR_SIZE+1];
        char m_TxDigis[(ADDR_SIZE*8)+1];
        char m_sTxCoordinates[COORDINATES_SIZE+1];
        char m_sTxStnType[STN_SIZE+1];

        char m_szRxToCall[12];
        char m_szRxFmCall[12];
        char m_szTxFmCall[12];

        char m_szRxBuffer[AX25_FRAME_MAX_SIZE+1];
        char * m_pRxBuffer;
        int  m_nRxDataType;
        position m_pos;
        BOOL m_bPosition;
        BOOL m_bCompressedData;

        char m_szExtraText[STRING_SIZE+1];

        int m_nDecodeType;

        char m_sIDigi[AX25_CALLSSID_SIZE+1][8];
        int m_nIGateCnt;
        int m_nIGateIndex;

};


#endif
