/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// aprsqth.cpp
//

#define APRSQTH_CPP

#include <string.h>

// libwxn
#include <textfile.h>
#include <trim.h>
#include <strxcpy.h>
#include <makeuc.h>

#include "aprsqth.h"


//////////////////////////////////////////////////////////////////////////////
// AprsQth - find entry in file <aprsqth>. Returns TRUE if found, FALSE
//           otherwise. Used to fill record entries for APRS nodes.
//           Comment lines begin with ';' and are ignored. File format
//           follows:
//
//              <call-ssid>, <city>, <county>, <state>
//

BOOL AprsQth( noderef * pInfo )
{
    textfile file;
    char * ptr;
    char szString[STRING_SIZE];

    if( !file.open( WXN_SYSCONFDIR "aprsqth", FILE_READ ) )
        return FALSE;

    //FOREVER
    for(;;)
    {
        //////////////////////////// node header
        if( fgets( szString, STRING_SIZE, file.handle() ) == NULL )
            break;

        // skip comments
        if( *szString == '#' )
            continue;

        // make ASCIIZ
        ptr = strchr( szString, '\r' );
        if( ptr )
            *ptr = 0;
        ptr = strchr( szString, '\n' );
        if( ptr )
            *ptr = 0;

        ptr = strtok( szString, "," );
        if( ptr )
        {
            TrimSpaces( ptr );
            if( strcasecmp( pInfo->szCallSsid, ptr ) )
                continue;
        }
        else
            return FALSE;

        ptr = strtok( NULL, "," );
        if( ptr )
        {
            TrimSpaces( ptr );
            strxcpy( pInfo->szCity, ptr, CITY_SIZE );
        }
        else
            return FALSE;

        ptr = strtok( NULL, "," );
        if( ptr )
        {
            TrimSpaces( ptr );
            strxcpy( pInfo->szCounty, ptr, COUNTY_SIZE );
        }
        else
            return FALSE;

        ptr = strtok( NULL, "," );
        if( ptr )
        {
            TrimSpaces( ptr );
            makeuc( ptr );
            strxcpy( pInfo->szState, ptr, STATE_SIZE );
        }
        else
            return FALSE;

        return TRUE;
    }

    return FALSE;
}

