/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// callbook.cpp - class interface to callbook table
//
//                           -------------------
//  begin                : Sat Dec 17 2005
//  copyright            : (C) 2005 by John Bennett
//  email                : n4xi@arrl.net
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
////////////////////////////////////////////////////////////////////////////

#define _CALLBOOK_CPP_

#include <string.h>
#include <ctype.h>
#include <stdlib.h>

// libwxn
#include <makeuc.h>
#include <titlecase.h>
#include <strxcpy.h>
#include <libpq-fe.h>

#include "callbook.h"


/////////////////////////////////////////////////////////////////////////////
// Constructor
//

Callbook::Callbook()
{
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
//

Callbook::~Callbook()
{
}


/////////////////////////////////////////////////////////////////////////////
// QthLookup - lookup a call in the table.  Returns TRUE if found and
// <city, state/province> variables are filled, FALSE otherwise.
//

#define TEMP_SIZE 64    // for buffer size below

BOOL Callbook::Lookup( noderef * pInfo )
{
    PGconn * pConn;
    PGresult * pResult;
    char szExec[384];
    char * pS;
    char szCall[CALLSSID_SIZE+1];

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    strcpy( szCall, pInfo->szCallSsid );
    pS = strchr( szCall, '-' );
    if( pS )
        *pS = 0;    // remove trailing SSID if it exists
    makeuc( szCall );

    sprintf( szExec, "SELECT city,state FROM callbook WHERE callsign='%s'", szCall );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        strxcpy( pInfo->szCity, PQgetvalue( pResult, 0, 0 ), CITY_SIZE );
        strxcpy( pInfo->szState, PQgetvalue( pResult, 0, 1 ), STATE_SIZE );
        PQclear( pResult );
        PQfinish( pConn );
        return TRUE;
    }

    // else

    PQclear( pResult );
    PQfinish( pConn );
    return FALSE;

}

