/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// deny.cpp
//

#define _DENY_CPP_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// libwxn
#include <textfile.h>
#include <common.h>
#include <trim.h>
#include <makeuc.h>
#include <strxcpy.h>

#include "deny.h"


/////////////////////////////////////////////////////////////////////////////
//  Deny - used to determine if <call-ssid> should be excluded from database
//

deny::deny( void )
{
    m_pStart = NULL;
}

deny::~deny()
{
    clear();
}

BOOL deny::init( const char * szFileName )
{
    textfile file;
    deny_list * pNew;
    deny_list * pLast;
    char * pS;
    char szString[STRING_SIZE];

    sprintf( szString, WXN_SYSCONFDIR "%s.deny", szFileName );
    if( !file.open( szString, FILE_READ ) )
        return FALSE;

    clear();
    pLast = NULL;

    //FOREVER
    for(;;)
    {
        //////////////////////////// get entries in 'deny' file until all done

        if( fgets( szString, STRING_SIZE, file.handle() ) == NULL )
            break;

        // skip comments
        if( *szString == '#' )
            continue;

        // get rid of CR-LF's
        pS = strchr( szString, '\n' );
        if( pS )
            *pS = 0;
        pS = strchr( szString, '\r' );
        if( pS )
            *pS = 0;

        TrimSpaces( szString );

        if( strlen( szString ) == 0 )
            continue;

        // add to list
        pNew = new deny_list;
        if( pNew == NULL )
            return FALSE;
        if( m_pStart == NULL )
            m_pStart = pNew;
        pNew->pNext = NULL;
        if( pLast )
            pLast->pNext = pNew;
        pLast = pNew;

        makeuc( szString );
        strxcpy( pNew->sNode, szString, 31 );
    }

    return TRUE;

}


BOOL deny::check( char * pCall )
{
    deny_list * pList;

    pList = m_pStart;
    while( pList )
    {
        if( strcmp( pList->sNode, pCall ) == 0 )
            return TRUE;
        pList = pList->pNext;
    }

    return FALSE;
}


BOOL deny::status( void )
{
    if( m_pStart )
        return TRUE;
    else
        return FALSE;
}


void deny::clear( void )
{
    deny_list * pList;
    deny_list * pFree;

    if( m_pStart == NULL )
        return;

    pList = m_pStart;
    while( pList )
    {
        pFree = pList;
        pList = pList->pNext;
        delete pFree;
    }

    m_pStart = NULL;
}

