/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// node.cpp  - handles I/O from database backend
//

#define _NODE_CPP_

#include <stdlib.h>
#include <memory.h>
#include <libpq-fe.h>

// libwxn
#include <tz.h>
#include <logmsg.h>

#include "node.h"
#include "aprsproc.h"


//////////////////////////////////////////////////////////////////////////////
// ReadNode - loads record from backend if szCallSsid exists. Returns TRUE
// success, FALSE on failure.
//

BOOL ReadNode( noderx * pBlock )
{
    // be sure to read station type and set config accordingly

    PGconn * pConn;
    PGresult * pResult;
    char szExec[384];
    char c;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    //////////////////////////////////////////// retrieve node info

    sprintf( szExec, "SELECT alias,city,county,state,fips,afos,latitude,longitude,timezone,"
        "elevation,haat,aprs_soft,stn_type,dst,dtg_local FROM nodes WHERE call_ssid='%s'",
        pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        strcpy( pBlock->Info.szAlias, PQgetvalue( pResult, 0, 0 ) );
        strcpy( pBlock->Info.szCity, PQgetvalue( pResult, 0, 1 ) );
        strcpy( pBlock->Info.szCounty, PQgetvalue( pResult, 0, 2 ) );
        strcpy( pBlock->Info.szState, PQgetvalue( pResult, 0, 3 ) );
        strcpy( pBlock->Info.szFips, PQgetvalue( pResult, 0, 4 ) );
        strcpy( pBlock->Info.szAfos, PQgetvalue( pResult, 0, 5 ) );
        pBlock->Info.dLatitude = atof( PQgetvalue( pResult, 0, 6 ) );
        pBlock->Info.dLongitude = atof( PQgetvalue( pResult, 0, 7 ) );
        pBlock->Info.nTimeZone = atoi( PQgetvalue( pResult, 0, 8 ) );
        pBlock->Info.nElevation = atoi( PQgetvalue( pResult, 0, 9 ) );
        pBlock->Info.nHAAT = atoi( PQgetvalue( pResult, 0, 10 ) );
        pBlock->Info.nAprSoftType = atoi( PQgetvalue( pResult, 0, 11 ) );
        pBlock->Info.nStnType = atoi( PQgetvalue( pResult, 0, 12 ) );
        c = *PQgetvalue( pResult, 0, 13 );
        if( c == 't' )
            pBlock->Info.bDst = TRUE;
        else
            pBlock->Info.bDst = FALSE;
        pBlock->Info.tUpdate = g_Time.dtg( PQgetvalue( pResult, 0, 14 ) );
    }
    else
    {
        PQclear( pResult );
        PQfinish( pConn );
        return FALSE;
    }
    PQclear( pResult );

    //////////////////////////////////////////// retrieve data

    // temperature
    sprintf( szExec, "SELECT tcr, thr_val, thr_dtg, tlr_val, tlr_dtg, trd, trh, tyh, tyl "
        "FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        pBlock->Data.tcr = atoi( PQgetvalue( pResult, 0, 0 ) );
        pBlock->Data.thr.val = atoi( PQgetvalue( pResult, 0, 1 ) );
        pBlock->Data.thr.dtg = g_Time.dtg( PQgetvalue( pResult, 0, 2 ) );
        pBlock->Data.tlr.val = atoi( PQgetvalue( pResult, 0, 3 ) );
        pBlock->Data.tlr.dtg = g_Time.dtg( PQgetvalue( pResult, 0, 4 ) );
        pBlock->Data.trd = atoi( PQgetvalue( pResult, 0, 5 ) );
        pBlock->Data.trh = atoi( PQgetvalue( pResult, 0, 6 ) );
        pBlock->Data.tyh = atoi( PQgetvalue( pResult, 0, 7 ) );
        pBlock->Data.tyl = atoi( PQgetvalue( pResult, 0, 8 ) );
    }
    PQclear( pResult );

    ////////////// wind
    // current
    sprintf( szExec, "SELECT wgc_spd, wgc_dir, wac_spd, wac_dir "
        "FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        pBlock->Data.wgc.spd = atoi( PQgetvalue( pResult, 0, 0 ) );
        pBlock->Data.wgc.dir = atoi( PQgetvalue( pResult, 0, 1 ) );
        pBlock->Data.wac.spd = atoi( PQgetvalue( pResult, 0, 2 ) );
        pBlock->Data.wac.dir = atoi( PQgetvalue( pResult, 0, 3 ) );
    }
    PQclear( pResult );

    // high gust
    sprintf( szExec, "SELECT wgh_spd, wgh_dir, wgh_dtg FROM nodes WHERE call_ssid='%s'",
        pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        pBlock->Data.wgh.spd = atoi( PQgetvalue( pResult, 0, 0 ) );
        pBlock->Data.wgh.dir = atoi( PQgetvalue( pResult, 0, 1 ) );
        pBlock->Data.wgh.dtg = g_Time.dtg( PQgetvalue( pResult, 0, 2 ) );
    }
    PQclear( pResult );

    // avg hi/lo
    sprintf( szExec, "SELECT wah_spd, wah_dir, wah_dtg, wal_spd, wal_dir, wal_dtg "
        "FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        pBlock->Data.wah.spd = atoi( PQgetvalue( pResult, 0, 0 ) );
        pBlock->Data.wah.dir = atoi( PQgetvalue( pResult, 0, 1 ) );
        pBlock->Data.wah.dtg = g_Time.dtg( PQgetvalue( pResult, 0, 2 ) );
        pBlock->Data.wal.spd = atoi( PQgetvalue( pResult, 0, 3 ) );
        pBlock->Data.wal.dir = atoi( PQgetvalue( pResult, 0, 4 ) );
        pBlock->Data.wal.dtg = g_Time.dtg( PQgetvalue( pResult, 0, 5 ) );
    }
    PQclear( pResult );

    // wind chill
    sprintf( szExec, "SELECT wca, wcg FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        pBlock->Data.wca = atoi( PQgetvalue( pResult, 0, 0 ) );
        pBlock->Data.wcg = atoi( PQgetvalue( pResult, 0, 1 ) );
    }
    PQclear( pResult );

    // precip
    sprintf( szExec, "SELECT pca, pri, prh, plh, p24, pty, ptm, pta "
        "FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        pBlock->Data.pca = atoi( PQgetvalue( pResult, 0, 0 ) );
        pBlock->Data.pri = atoi( PQgetvalue( pResult, 0, 1 ) );
        pBlock->Data.prh = atoi( PQgetvalue( pResult, 0, 2 ) );
        pBlock->Data.plh = atoi( PQgetvalue( pResult, 0, 3 ) );
        pBlock->Data.p24 = atoi( PQgetvalue( pResult, 0, 4 ) );
        pBlock->Data.pty = atoi( PQgetvalue( pResult, 0, 5 ) );
        pBlock->Data.ptm = atoi( PQgetvalue( pResult, 0, 6 ) );
        pBlock->Data.pta = atoi( PQgetvalue( pResult, 0, 7 ) );
    }
    PQclear( pResult );

    // barometer
    sprintf( szExec, "SELECT bcr, bhr_val, bhr_dtg, blr_val, blr_dtg, brd, brh "
        "FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        if( !PQgetisnull( pResult, 0, 0 ) )
        {
            pBlock->Data.bcr = atoi( PQgetvalue( pResult, 0, 0 ) );
            pBlock->Data.bhr.val = atoi( PQgetvalue( pResult, 0, 1 ) );
            pBlock->Data.bhr.dtg = g_Time.dtg( PQgetvalue( pResult, 0, 2 ) );
            pBlock->Data.blr.val = atoi( PQgetvalue( pResult, 0, 3 ) );
            pBlock->Data.blr.dtg = g_Time.dtg( PQgetvalue( pResult, 0, 4 ) );
            pBlock->Data.brd = atoi( PQgetvalue( pResult, 0, 5 ) );
            pBlock->Data.brh = atoi( PQgetvalue( pResult, 0, 6 ) );
        }
    }
    PQclear( pResult );

    // humidity
    sprintf( szExec, "SELECT hcr, hhr_val, hhr_dtg, hlr_val, hlr_dtg, hrd, hrh "
        "FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        if( !PQgetisnull( pResult, 0, 0 ) )
        {
            pBlock->Data.hcr = atoi( PQgetvalue( pResult, 0, 0 ) );
            pBlock->Data.hhr.val = atoi( PQgetvalue( pResult, 0, 1 ) );
            pBlock->Data.hhr.dtg = g_Time.dtg( PQgetvalue( pResult, 0, 2 ) );
            pBlock->Data.hlr.val = atoi( PQgetvalue( pResult, 0, 3 ) );
            pBlock->Data.hlr.dtg = g_Time.dtg( PQgetvalue( pResult, 0, 4 ) );
            pBlock->Data.hrd = atoi( PQgetvalue( pResult, 0, 5 ) );
            pBlock->Data.hrh = atoi( PQgetvalue( pResult, 0, 6 ) );
        }
    }
    PQclear( pResult );

    // emp
    sprintf( szExec, "SELECT ecc, eqc, exc, ehc, edc "
        "FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        if( !PQgetisnull( pResult, 0, 0 ) )
        {
            pBlock->Data.ecc = (unsigned) atol( PQgetvalue( pResult, 0, 0 ) );
            pBlock->Data.eqc = (unsigned) atol( PQgetvalue( pResult, 0, 1 ) );
            pBlock->Data.exc = (unsigned) atol( PQgetvalue( pResult, 0, 2 ) );
            pBlock->Data.ehc = (unsigned) atol( PQgetvalue( pResult, 0, 3 ) );
            pBlock->Data.edc = (unsigned) atol( PQgetvalue( pResult, 0, 4 ) );
        }
    }
    PQclear( pResult );

    // radiation
    sprintf( szExec, "SELECT rca, rha, rda, rmax, ralarm_count, ralarm_trip "
        "FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        if( !PQgetisnull( pResult, 0, 0 ) )
        {
            pBlock->Data.rca = atoi( PQgetvalue( pResult, 0, 0 ) );
            pBlock->Data.rha = atoi( PQgetvalue( pResult, 0, 1 ) );
            pBlock->Data.rda = atoi( PQgetvalue( pResult, 0, 2 ) );
            pBlock->Data.rmax = atoi( PQgetvalue( pResult, 0, 3 ) );
            pBlock->Data.ralarm_count = atoi( PQgetvalue( pResult, 0, 4 ) );
            pBlock->Info.nRadAlarmTrip = atoi( PQgetvalue( pResult, 0, 5 ) );
        }
    }
    PQclear( pResult );

    // sunshine
    sprintf( szExec, "SELECT sun_now, sun_min FROM nodes WHERE call_ssid='%s'",
        pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        if( !PQgetisnull( pResult, 0, 0 ) )
        {
            pBlock->Data.bSunNow = atoi( PQgetvalue( pResult, 0, 0 ) );
            pBlock->Data.nSunMin = atoi( PQgetvalue( pResult, 0, 1 ) );
        }
    }
    PQclear( pResult );

    // soil temp at 6" depth
    sprintf( szExec, "SELECT tsd FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        if( !PQgetisnull( pResult, 0, 0 ) )
            pBlock->Data.tsd = atoi( PQgetvalue( pResult, 0, 0 ) );
    }
    PQclear( pResult );

    // inside temp
    sprintf( szExec, "SELECT ist FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        if( !PQgetisnull( pResult, 0, 0 ) )
            pBlock->Data.ist = atoi( PQgetvalue( pResult, 0, 0 ) );
    }
    PQclear( pResult );

    // inside humidity
    sprintf( szExec, "SELECT ish FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        if( !PQgetisnull( pResult, 0, 0 ) )
            pBlock->Data.ish = atoi( PQgetvalue( pResult, 0, 0 ) );
    }
    PQclear( pResult );

    PQfinish( pConn );

    return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// NodeExists - checks for existance of node in database. Returns TRUE
// if entry exists, FALSE if not.
//

BOOL NodeExists( noderx * pBlock )
{
    char szExec[384];
    PGconn * pConn;
    PGresult * pResult;
    BOOL bStatus;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    sprintf( szExec, "SELECT call_ssid FROM nodes WHERE call_ssid='%s'", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( PQresultStatus( pResult ) != PGRES_TUPLES_OK )
    {
        PQclear( pResult );
        PQfinish( pConn );
        return FALSE;
    }
    if( PQntuples( pResult ) == 0 )
        bStatus = FALSE;    // record does not exist
    else
        bStatus = TRUE;     // found

    PQclear( pResult );
    PQfinish( pConn );

    return bStatus;
}


//////////////////////////////////////////////////////////////////////////////
// CreateRecord - creates record for <call-ssid> and writes gdb info to
// backend. Returns TRUE success, FALSE on failure.
//

BOOL CreateRecord( noderx * pBlock )
{
    char szExec[384];
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    sprintf( szExec, "INSERT INTO nodes( call_ssid ) VALUES( '%s' )", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( PQresultStatus( pResult ) != PGRES_COMMAND_OK )
    {
        PQclear( pResult );
        PQfinish( pConn );
        sprintf( szExec, "[%s] in [nodes] table", pBlock->Info.szCallSsid );
        g_log.PostMsg( APRMSG_CREATE_RECORD_FAIL, szExec );
        return FALSE;
    }
    PQclear( pResult );

    pResult = PQexec( pConn, "BEGIN work" );
    PQclear( pResult );

    if( strcmp( pBlock->Info.szCity, g_szUnknownQth ) ) // QTH *has* been resolved
    {
        sprintf( szExec, "UPDATE nodes SET city='%s', county='%s', state='%s', alias='%s', "
            "longitude=%f, latitude=%f, timezone=%i, dst='%c', afos='%s', aprs_soft=%i, stn_type=%i, fips='%s' "
            "WHERE call_ssid='%s'",
            pBlock->Info.szCity, pBlock->Info.szCounty, pBlock->Info.szState, pBlock->Info.szAlias,
            pBlock->Info.dLongitude, pBlock->Info.dLatitude, pBlock->Info.nTimeZone,
            pBlock->Info.bDst ? 't' : 'f', pBlock->Info.szAfos, pBlock->Info.nAprSoftType,
            pBlock->Info.nStnType, pBlock->Info.szFips,
            pBlock->Info.szCallSsid );
    }
    else
    {
        sprintf( szExec, "UPDATE nodes SET city='%s', alias='%s', longitude=%f, latitude=%f, aprs_soft=%i, "
            "stn_type=%i WHERE call_ssid='%s'",
            pBlock->Info.szCity, pBlock->Info.szAlias, pBlock->Info.dLongitude, pBlock->Info.dLatitude,
            pBlock->Info.nAprSoftType, pBlock->Info.nStnType, pBlock->Info.szCallSsid );
    }
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    if( pBlock->Info.nElevation )
    {
        sprintf( szExec, "UPDATE nodes SET elevation=%i WHERE call_ssid='%s'",
            pBlock->Info.nElevation, pBlock->Info.szCallSsid );
        PQexec( pConn, szExec );
        PQclear( pResult );
    }

    if( pBlock->Info.nHAAT )
    {
        sprintf( szExec, "UPDATE nodes SET haat=%i WHERE call_ssid='%s'",
            pBlock->Info.nHAAT, pBlock->Info.szCallSsid );
        PQexec( pConn, szExec );
        PQclear( pResult );
    }

    pResult = PQexec( pConn, "COMMIT work" );
    PQclear( pResult );

    // create receive time record - time will be added by WriteBase()
    sprintf( szExec, "INSERT INTO rcvtimes( call_ssid ) VALUES( '%s' )", pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( PQresultStatus( pResult ) != PGRES_COMMAND_OK )
    {
        sprintf( szExec, "[%s] in [rcvtimes] table", pBlock->Info.szCallSsid );
        g_log.PostMsg( APRMSG_CREATE_RECORD_FAIL, szExec );
    }
    PQclear( pResult );

    PQfinish( pConn );

    return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// ReadZoneInfo - reads timezone info for node from backend. Returns TRUE
// success, FALSE on failure.
//

BOOL ReadZoneInfo( noderx * pBlock )
{
    char szExec[384];
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    sprintf( szExec, "SELECT latitude,longitude,timezone,dst FROM nodes WHERE call_ssid='%s'",
        pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && (PQntuples( pResult ) > 0) )
    {
        pBlock->Info.dLatitude = atof( PQgetvalue( pResult, 0, 0) );
        pBlock->Info.dLongitude = atof( PQgetvalue( pResult, 0, 1) );
        pBlock->Info.nTimeZone = atoi( PQgetvalue( pResult, 0, 2) );
        if( *PQgetvalue( pResult, 0, 3 ) == 't' )
            pBlock->Info.bDst = TRUE;
        else
            pBlock->Info.bDst = FALSE;
        PQclear( pResult );
        PQfinish( pConn );
        return TRUE;
    }

    // else
    PQclear( pResult );
    PQfinish( pConn );
    return FALSE;
}


//////////////////////////////////////////////////////////////////////////////
// WriteBase - writes native APRS record to backend. Returns TRUE success,
// FALSE on failure.
//

BOOL WriteBase( noderx * pBlock )
{
    char szExec[384];
    char szLocal[32];
    char szZulu[32];
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    pResult = PQexec( pConn, "BEGIN work" );
    PQclear( pResult );

    sprintf( szExec, "UPDATE nodes SET aprs_soft=%i, stn_type=%i WHERE call_ssid='%s'",
            pBlock->Info.nAprSoftType, pBlock->Info.nStnType, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    g_Time.MakeNodeTimestamp( szLocal, pBlock->Info.nTimeZone, pBlock->Info.bDst );
    g_Time.CurrentZuluTimestamp( szZulu );
    sprintf( szExec, "UPDATE nodes SET tcr=%i, plh=%i, pca=%i, p24=%i, "
        "wgc_spd=%i, wgc_dir= -1, wac_spd=%i, wac_dir=%i, dtg_local='%s', dtg_zulu='%s' "
        "WHERE call_ssid='%s'",
            pBlock->Data.tcr, pBlock->Data.plh, pBlock->Data.pca, pBlock->Data.p24,
            pBlock->Data.wgc.spd,
            pBlock->Data.wac.spd, pBlock->Data.wac.dir,
            szLocal, szZulu,
            pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    if( pBlock->Data.bcr )
        sprintf( szExec, "UPDATE nodes SET bcr=%i WHERE call_ssid='%s'",
            pBlock->Data.bcr, pBlock->Info.szCallSsid );
    else
        sprintf( szExec, "UPDATE nodes SET bcr=NULL WHERE call_ssid='%s'",
            pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    if( pBlock->Data.hcr )
        sprintf( szExec, "UPDATE nodes SET hcr=%i WHERE call_ssid='%s'",
            pBlock->Data.hcr, pBlock->Info.szCallSsid );
    else
        sprintf( szExec, "UPDATE nodes SET hcr=NULL WHERE call_ssid='%s'",
            pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    if( pBlock->Info.dLatitude || pBlock->Info.dLongitude )
    {
        sprintf( szExec, "UPDATE nodes SET latitude=%f, longitude=%f WHERE call_ssid='%s'",
            pBlock->Info.dLatitude, pBlock->Info.dLongitude, pBlock->Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // timestamp receipt
    g_Time.CurrentZuluTimestamp( szZulu );
    sprintf( szExec, "UPDATE rcvtimes SET aprs_time='%s' WHERE call_ssid='%s'",
        szZulu, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    pResult = PQexec( pConn, "COMMIT work" );
    PQclear( pResult );

    PQfinish( pConn );

    return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// WriteCalc - writes calculated fields to backend. Returns TRUE success,
// FALSE on failure.
//

BOOL WriteCalc( noderx * pBlock )
{
    char szExec[384];
    char szRiseTime[32];
    char szSetTime[32];
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    pResult = PQexec( pConn, "BEGIN work" );
    PQclear( pResult );

    if( pBlock->Data.hcr )
    {
        if( pBlock->Data.nHeatIndex )
            sprintf( szExec, "UPDATE nodes SET fog='%c', heatindex=%i, dewpoint=%i "
                "WHERE call_ssid='%s'",
                    pBlock->Data.bFog ? 't':'f', pBlock->Data.nHeatIndex, pBlock->Data.nDewPoint,
                    pBlock->Info.szCallSsid );
        else
            sprintf( szExec, "UPDATE nodes SET fog='%c', heatindex=NULL, dewpoint=%i "
                "WHERE call_ssid='%s'",
                    pBlock->Data.bFog ? 't':'f', pBlock->Data.nDewPoint,
                    pBlock->Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    if( pBlock->Data.tcr == pBlock->Data.wcg )
        // if current temp equals wind chill, there is no wind chill
        sprintf( szExec, "UPDATE nodes SET wca=NULL, wcg=NULL WHERE call_ssid='%s'",
            pBlock->Info.szCallSsid );
    else
        sprintf( szExec, "UPDATE nodes SET wca=%i, wcg=%i WHERE call_ssid='%s'",
            pBlock->Data.wca, pBlock->Data.wcg, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    if( pBlock->Data.tSunrise )
    {
        g_Time.MakeRiseSet( pBlock->Data.tSunrise, szRiseTime );
        g_Time.MakeRiseSet( pBlock->Data.tSunset, szSetTime );
        sprintf( szExec, "UPDATE nodes SET sunrise='%s', sunset='%s' WHERE call_ssid='%s'",
            szRiseTime, szSetTime, pBlock->Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    pResult = PQexec( pConn, "COMMIT work" );
    PQclear( pResult );

    PQfinish( pConn );

    return TRUE;
}



//////////////////////////////////////////////////////////////////////////////
// WriteTypeA - writes WXN extended weather data type A to backend. Returns
// TRUE success and writes receive time to backend, FALSE on failure.
//
//  fips, city
//  alias
//  temp: now, hi, hi time, low, low time, hr rate, 24 hr rate
//  yday temps: hi, lo
//

BOOL WriteTypeA( noderx * pBlock )
{
    char szTime1[32];
    char szTime2[32];
    char szExec[384];
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    pResult = PQexec( pConn, "BEGIN work" );
    PQclear( pResult );

    g_Time.MakeLocalTimestamp( szTime1, pBlock->Data.thr.dtg );
    g_Time.MakeLocalTimestamp( szTime2, pBlock->Data.tlr.dtg );
    if( pBlock->Data.tsd )
        sprintf( szExec, "UPDATE nodes SET fips='%s', city='%s', state='%s', alias ='%s', "
            "tcr=%i, thr_val=%i, thr_dtg='%s', tlr_val=%i, tlr_dtg='%s', "
            "trh=%i, trd=%i, tyh=%i, tyl=%i, tsd=%i, cool_days=%i, heat_days=%i WHERE call_ssid='%s'",
            pBlock->Info.szFips, pBlock->Info.szCity, pBlock->Info.szState, pBlock->Info.szAlias,
            pBlock->Data.tcr, pBlock->Data.thr.val, szTime1, pBlock->Data.tlr.val, szTime2,
            pBlock->Data.trh, pBlock->Data.trd, pBlock->Data.tyh, pBlock->Data.tyl, pBlock->Data.tsd,
            pBlock->Data.nCoolDays, pBlock->Data.nHeatDays, pBlock->Info.szCallSsid );
    else
        sprintf( szExec, "UPDATE nodes SET fips='%s', city='%s', state='%s', alias ='%s', "
            "tcr=%i, thr_val=%i, thr_dtg='%s', tlr_val=%i, tlr_dtg='%s', "
            "trh=%i, trd=%i, tyh=%i, tyl=%i, cool_days=%i, heat_days=%i WHERE call_ssid='%s'",
            pBlock->Info.szFips, pBlock->Info.szCity, pBlock->Info.szState, pBlock->Info.szAlias,
            pBlock->Data.tcr, pBlock->Data.thr.val, szTime1, pBlock->Data.tlr.val, szTime2,
            pBlock->Data.trh, pBlock->Data.trd, pBlock->Data.tyh, pBlock->Data.tyl,
            pBlock->Data.nCoolDays, pBlock->Data.nHeatDays, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    g_Time.MakeRiseSet( pBlock->Data.tSunrise, szTime1 );
    g_Time.MakeRiseSet( pBlock->Data.tSunset, szTime2 );
    sprintf( szExec, "UPDATE nodes SET sunrise='%s', sunset='%s' WHERE call_ssid='%s'",
        szTime1, szTime2, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    pResult = PQexec( pConn, "COMMIT work" );
    PQclear( pResult );

    // timestamp receipt
    g_Time.CurrentZuluTimestamp( szTime1 );
    sprintf( szExec, "UPDATE rcvtimes SET a_time='%s' WHERE call_ssid='%s'",
        szTime1, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    PQfinish( pConn );

    return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// WriteTypeB - writes WXN extended weather data type B to backend. Returns
// TRUE success and writes receive time to backend, FALSE on failure.
//
//  pca, pri, prh
//  plh, p24, pty, ptm, pta
//  ecc, eqc, exc, ehc, edc
//

BOOL WriteTypeB( noderx * pBlock )
{
    char szTime[32];
    char szExec[384];
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    pResult = PQexec( pConn, "BEGIN work" );
    PQclear( pResult );

    sprintf( szExec, "UPDATE nodes SET pca=%i, pri=%i, prh=%i, plh=%i, p24=%i, pty=%i, "
        "ptm=%i, pta=%i WHERE call_ssid='%s'",
        pBlock->Data.pca, pBlock->Data.pri, pBlock->Data.prh, pBlock->Data.plh, pBlock->Data.p24,
        pBlock->Data.pty, pBlock->Data.ptm, pBlock->Data.pta, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // emp
    if( pBlock->Data.edc )
    {
        sprintf( szExec, "UPDATE nodes SET ecc=%i, eqc=%i, exc=%i, ehc=%i, edc=%i WHERE call_ssid='%s'",
            pBlock->Data.ecc, pBlock->Data.eqc, pBlock->Data.exc, pBlock->Data.ehc, pBlock->Data.edc,
            pBlock->Info.szCallSsid );
    }
    else
    {
        sprintf( szExec, "UPDATE nodes SET ecc=NULL, eqc=NULL, exc=NULL, ehc=NULL, edc=NULL WHERE call_ssid='%s'",
            pBlock->Info.szCallSsid );
    }
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // timestamp receipt
    g_Time.CurrentZuluTimestamp( szTime );
    sprintf( szExec, "UPDATE rcvtimes SET b_time='%s' WHERE call_ssid='%s'",
        szTime, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    pResult = PQexec( pConn, "COMMIT work" );
    PQclear( pResult );

    PQfinish( pConn );

    return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// WriteTypeC - writes WXN extended weather data type C to backend. Returns
// TRUE success and writes receive time to backend, FALSE on failure.
//
//  wind:
//      wac_spd, wac_dir, wgc_spd, wgc_dir,
//      wah_spd, wah_dir, wah_dtg, wal_spd, wal_dir, wal_dtg
//      wgh_spd, wgh_dir, wgh_dtg
//      haat
//

BOOL WriteTypeC( noderx * pBlock )
{
    char szHiTime[32];
    char szLoTime[32];
    char szExec[384];
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    pResult = PQexec( pConn, "BEGIN work" );
    PQclear( pResult );

    // current
    sprintf( szExec, "UPDATE nodes SET wac_spd=%i, wac_dir=%i, wgc_spd=%i, wgc_dir=%i "
        "WHERE call_ssid='%s'",
        pBlock->Data.wac.spd, pBlock->Data.wac.dir, pBlock->Data.wgc.spd, pBlock->Data.wgc.dir,
        pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // avg wind
    g_Time.MakeLocalTimestamp( szHiTime, pBlock->Data.wah.dtg );
    g_Time.MakeLocalTimestamp( szLoTime, pBlock->Data.wal.dtg );
    sprintf( szExec, "UPDATE nodes SET wah_spd=%i, wah_dir=%i, wah_dtg='%s', "
        "wal_spd=%i, wal_dir=%i, wal_dtg='%s' WHERE call_ssid='%s'",
        pBlock->Data.wah.spd, pBlock->Data.wah.dir, szHiTime,
        pBlock->Data.wal.spd, pBlock->Data.wal.dir, szLoTime, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // gusts
    g_Time.MakeLocalTimestamp( szHiTime, pBlock->Data.wgh.dtg );
    sprintf( szExec, "UPDATE nodes SET wgh_spd=%i, wgh_dir=%i, wgh_dtg='%s'  WHERE call_ssid='%s'",
        pBlock->Data.wgh.spd, pBlock->Data.wgh.dir, szHiTime, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // haat
    if( pBlock->Info.nHAAT )
    {
        sprintf( szExec, "UPDATE nodes SET haat=%i WHERE call_ssid='%s'",
            pBlock->Info.nHAAT, pBlock->Info.szCallSsid );
    }
    else
    {
        sprintf( szExec, "UPDATE nodes SET haat=NULL WHERE call_ssid='%s'",
            pBlock->Info.szCallSsid );
    }
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // timestamp receipt
    g_Time.CurrentZuluTimestamp( szHiTime );
    sprintf( szExec, "UPDATE rcvtimes SET c_time='%s' WHERE call_ssid='%s'",
        szHiTime, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    pResult = PQexec( pConn, "COMMIT work" );
    PQclear( pResult );

    PQfinish( pConn );

    return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// WriteTypeD - writes WXN extended weather data type D to backend. Returns
// TRUE success and writes receive time to backend, FALSE on failure.
//
//  bar:
//      bcr, bhr_val, bhr_dtg, blr_val, blr_dtg, brh, brd
//  humid:
//      hcr, hhr_val, hhr_dtg, hlr_val, hlr_dtg, hhr, hrd
//  inside:
//      ist, ish
//

BOOL WriteTypeD( noderx * pBlock )
{
    char szHiTime[32];
    char szLoTime[32];
    char szExec[384];
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    pResult = PQexec( pConn, "BEGIN work" );
    PQclear( pResult );

    // pressure
    if( pBlock->Data.bcr )
    {
        g_Time.MakeLocalTimestamp( szHiTime, pBlock->Data.bhr.dtg );
        g_Time.MakeLocalTimestamp( szLoTime, pBlock->Data.blr.dtg );
        sprintf( szExec, "UPDATE nodes SET bcr=%i, bhr_val=%i, bhr_dtg='%s', blr_val=%i, blr_dtg='%s', "
            "brh=%i, brd=%i WHERE call_ssid='%s'",
            pBlock->Data.bcr, pBlock->Data.bhr.val, szHiTime, pBlock->Data.blr.val, szLoTime,
            pBlock->Data.brh, pBlock->Data.brd, pBlock->Info.szCallSsid );
    }
    else
    {
        sprintf( szExec, "UPDATE nodes SET bcr=NULL, bhr_val=NULL, bhr_dtg=NULL, blr_val=NULL, blr_dtg=NULL, "
            "brh=NULL, brd=NULL WHERE call_ssid='%s'",
            pBlock->Info.szCallSsid );
    }
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // humidity
    if( pBlock->Data.hcr )
    {
        g_Time.MakeLocalTimestamp( szHiTime, pBlock->Data.hhr.dtg );
        g_Time.MakeLocalTimestamp( szLoTime, pBlock->Data.hlr.dtg );
        sprintf( szExec, "UPDATE nodes SET hcr=%i, hhr_val=%i, hhr_dtg='%s', hlr_val=%i, hlr_dtg='%s', "
            "hrh=%i, hrd=%i WHERE call_ssid='%s'",
            pBlock->Data.hcr, pBlock->Data.hhr.val, szHiTime, pBlock->Data.hlr.val, szLoTime,
            pBlock->Data.hrh, pBlock->Data.hrd, pBlock->Info.szCallSsid );
    }
    else
    {
        sprintf( szExec, "UPDATE nodes SET hcr=NULL, hhr_val=NULL, hhr_dtg=NULL, hlr_val=NULL, hlr_dtg=NULL, "
            "hrh=NULL, hrd=NULL WHERE call_ssid='%s'",
            pBlock->Info.szCallSsid );
    }
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // inside temperature
    if( pBlock->Data.ist )
    {
        sprintf( szExec, "UPDATE nodes SET ist=%i WHERE call_ssid='%s'",
            pBlock->Data.ist, pBlock->Info.szCallSsid );
    }
    else
    {
        sprintf( szExec, "UPDATE nodes SET ist=NULL WHERE call_ssid='%s'",
            pBlock->Info.szCallSsid );
    }
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // inside humidity
    if( pBlock->Data.ish )
    {
        sprintf( szExec, "UPDATE nodes SET ish=%i WHERE call_ssid='%s'",
            pBlock->Data.ish, pBlock->Info.szCallSsid );
    }
    else
    {
        sprintf( szExec, "UPDATE nodes SET ish=NULL WHERE call_ssid='%s'",
            pBlock->Info.szCallSsid );
    }
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // timestamp receipt
    g_Time.CurrentZuluTimestamp( szHiTime );
    sprintf( szExec, "UPDATE rcvtimes SET d_time='%s' WHERE call_ssid='%s'",
        szHiTime, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    pResult = PQexec( pConn, "COMMIT work" );
    PQclear( pResult );

    PQfinish( pConn );

    return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// WriteTypeE - writes WXN extended weather data type E to backend. Returns
// TRUE success and writes receive time to backend, FALSE on failure.
//
//      rca, rha, rda, rmax, rmax_count, rad_alarm_trip
//

BOOL WriteTypeE( noderx * pBlock )
{
    char szTime[32];
    char szExec[384];
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return FALSE;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return FALSE;
    }

    pResult = PQexec( pConn, "BEGIN work" );
    PQclear( pResult );

    if( pBlock->Data.rmax )
    {
        sprintf( szExec, "UPDATE nodes SET rca=%i, rha=%i, rda=%i, rmax=%i, ralarm_count=%i, "
            "ralarm_trip=%i WHERE call_ssid='%s'",
            pBlock->Data.rca, pBlock->Data.rha, pBlock->Data.rda, pBlock->Data.rmax, pBlock->Data.ralarm_count,
            pBlock->Info.nRadAlarmTrip, pBlock->Info.szCallSsid );
    }
    else
    {
        sprintf( szExec, "UPDATE nodes SET rca=NULL, rha=NULL, rda=NULL, rmax=NULL, ralarm_count=NULL, "
            "ralarm_trip=NULL WHERE call_ssid='%s'",
            pBlock->Info.szCallSsid );
    }
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // timestamp receipt
    g_Time.CurrentZuluTimestamp( szTime );
    sprintf( szExec, "UPDATE rcvtimes SET e_time='%s' WHERE call_ssid='%s'",
        szTime, pBlock->Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    pResult = PQexec( pConn, "COMMIT work" );
    PQclear( pResult );

    PQfinish( pConn );

    return TRUE;

}

