/***************************************************************************
 *   Copyright (C) 2005 by John Bennett                                    *
 *   n4xi@arrl.net                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// qrz.cpp - user account interface to QRZ database
//
//                           -------------------
//  begin                : Wed Feb  5 11:47:20 CST 2003
//  copyright            : (C) 2003 by John Bennett
//  email                : n4xi@arrl.net
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
////////////////////////////////////////////////////////////////////////////

#define _QRZ_CPP_

#include <string.h>
#include <ctype.h>
#include <stdlib.h>

// libwxn
#include <makeuc.h>
#include <titlecase.h>
#include <strxcpy.h>
//#include <logmsg.h>

#include "qrz.h"


/////////////////////////////////////////////////////////////////////////////
// Constructor
//

qrz::qrz()
{
    m_hIndex = NULL;
    m_hDbf = NULL;
    m_pKeybase = NULL;
    m_pBuffer = m_szBuffer;
    m_bEnabled = FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// Destructor
//

qrz::~qrz()
{
    if( m_hIndex )
        fclose( m_hIndex );

    if( m_hDbf )
        fclose( m_hDbf );

    if( m_pKeybase )
        delete [] m_pKeybase;
}


/////////////////////////////////////////////////////////////////////////////
// Init - opens index and database files. Initializes for operations.
// Returns TRUE success, FALSE failure.
//

BOOL qrz::Init( char * pPath )
{
    char szPath[64];
    char szIndex[] = "callbkc.idx";

    ////////////////////////////////// open index
    sprintf( szPath, "%s%s", pPath, szIndex );
    m_hIndex = fopen( szPath, "rb" );
    if( m_hIndex == NULL )
        return FALSE;

    if( fread( &m_RawHeader, sizeof(struct rawheader), 1, m_hIndex ) == 0 )
    {
        fclose( m_hIndex );
        m_hIndex = NULL;
        return FALSE;
    }

    ////////////////////////////////// open main database

    sprintf( szPath, "%s%s", pPath, m_RawHeader.datafile );

    m_Index.bytesperkey = atol( m_RawHeader.bytesperkey );
    m_Index.numkeys = atoi(m_RawHeader.numkeys);
    m_Index.keylen = atoi(m_RawHeader.keylen);

    m_nKeysize = m_Index.numkeys * m_Index.keylen;
    m_pKeybase = new char[m_nKeysize];
    if ( m_pKeybase == 0 )
    {
        fclose( m_hIndex );
        m_hIndex = NULL;
        return FALSE;
    }
    memset( m_pKeybase, 0, m_nKeysize );

    if ( fread( m_pKeybase, m_nKeysize, 1, m_hIndex ) == 0 )
    {
        fclose( m_hIndex );
        m_hIndex = NULL;
        delete [] m_pKeybase;
        m_pKeybase = NULL;
        return FALSE;
    }

    m_hDbf = fopen( szPath, "rb" );       // open data file
    if ( m_hDbf == NULL )
    {
        fclose( m_hIndex );
        m_hIndex = NULL;
        delete [] m_pKeybase;
        m_pKeybase = NULL;
        return FALSE;
    }

    m_bEnabled = TRUE;

    return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// QthLookup - lookup a call in the database.  Returns TRUE if found and
// <city, state> variables are filled, FALSE otherwise. Only used by APRS.
//

#define TEMP_SIZE 64    // for buffer size below

BOOL qrz::QthLookup( noderef * pInfo )
{
    char szTemp[TEMP_SIZE+1];
    char szCallSsid[CALLSSID_SIZE+1];
    char * pS;

    if( !m_bEnabled )
        return FALSE;

    strcpy( szCallSsid, pInfo->szCallSsid );
    makeuc( szCallSsid );
    if ( SetIndex( szCallSsid ) )
    {
        if( Search( szCallSsid ) )
        {
            m_pBuffer = m_szBuffer+7;           // call sign
            ParseField( NULL, TEMP_SIZE );      // last name
            ParseField( NULL, TEMP_SIZE );      // name suffix
            ParseField( NULL, TEMP_SIZE );      // first name
            ParseField( NULL, TEMP_SIZE );      // middle initial
            ParseField( NULL, TEMP_SIZE );      // DOB
            ParseField( NULL, TEMP_SIZE );      // effective date
            ParseField( NULL, TEMP_SIZE );      // expiration date
            ParseField( NULL, TEMP_SIZE );      // street address
            ParseField( szTemp, TEMP_SIZE );    // city
            // ';' delimits non-US stations in the city field.
            // If found, this field contains all address information.
            // At the moment, it is assumed we have a Canadian
            // station. Canadian provinces use two-letter abbreviations
            // the US does. Information parsed accordingly.
            pS = strchr( szTemp, ';' );
            if ( pS )   // not a US call
            {
                *pS = 0;    // take out the ';'
                TitleCase( szTemp );
                strxcpy( pInfo->szCity, szTemp, CITY_SIZE );
                pS++;
                while ( *pS == ' ' )
                    pS++;
                *(pS+2) = 0;
                makeuc( pS );
                strxcpy( pInfo->szState, pS, STATE_SIZE );
            }
            else
            {
                TitleCase( szTemp );
                strxcpy( pInfo->szCity, szTemp, CITY_SIZE );
                ParseField( szTemp, TEMP_SIZE );    // state
                strxcpy( pInfo->szState, szTemp, STATE_SIZE );
            }

            return TRUE;
        }
    }

    return FALSE;
}


//////////////////////////////////////////////////////////////////////////
// SetIndex - returns TRUE if successful seek, otherwise FALSE - the call
// was not found.

BOOL qrz::SetIndex( const char * szCall )
{
    int i;
    int nCmp;
    unsigned long offset;
    char szFmtCall[CALLSSID_SIZE+1];
    char szIdxCall[CALLSSID_SIZE+1];
    char *pIndex;

    strcpy( szFmtCall, szCall );
    FormatCall( szFmtCall );

    pIndex = m_pKeybase + m_Index.keylen;
    for( i=1; i<m_Index.numkeys; i++ )
    {
        memcpy( szIdxCall, pIndex, 6 );
        szIdxCall[6] = 0;
        FormatCall( szIdxCall );
        nCmp = strcmp( szIdxCall, szFmtCall );
        if( nCmp >= 0 )
        {
            if ( nCmp == 0 )
                offset = m_Index.bytesperkey * i;
            else
                offset = m_Index.bytesperkey * (i-1);
            if( fseek(m_hDbf, offset, SEEK_SET) == 0 )
                return TRUE;
            else
            {
                //g_log.PostMsg( APRMSG_QRZ_SETINDEX_FAIL, " @ fseek()" );
                return FALSE;
            }
        }

        pIndex += m_Index.keylen;
    }

    //g_log.PostMsg( APRMSG_QRZ_SETINDEX_FAIL, " not found" );

    return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// FormatCall - reconstruct the user call in format used by QRZ database.
// Routine modifies the string handed off in call directly.
//

void qrz::FormatCall( char * szCall )
{
    int i;
    char * pSrc;
    char * pDest;
    char szSuffix[4];
    char szPrefix[3];
    char szDigit[2];

    pSrc = szCall;

    // parse prefix
    pDest = szPrefix;
    *pDest++ = *pSrc++;   // always copy first character (may be number)
    if( isalpha( *pSrc ) || (*pSrc == ' ') )
        *pDest++ = *pSrc++;
    else
        *pDest++ = ' ';
    *pDest = 0;

    // parse digit
    pDest = szDigit;
    if( isdigit( *pSrc ) )
        *pDest++ = *pSrc++;
    else
        *pDest++ = ' ';
    *pDest = 0;

    // parse suffix
    strcpy( szSuffix, "   " );
    pDest = szSuffix;
    for( i=0; i<3; i++ )
    {
        if( isalpha( *pSrc ) )
            *pDest++ = *pSrc++;
        else
            break;
    }
    // do not terminate - must be 3 char in length

    // reconstruct indexed call
    strcpy( szCall, szSuffix );
    strcat( szCall, szDigit );
    strcat( szCall, szPrefix );
}


/////////////////////////////////////////////////////////////////////////////
// Search - Search the QRZ database starting at the current position
// going forward for given call until the next index is reached.
// Return TRUE if found, FALSE otherwise.

BOOL qrz::Search( const char * szCall )
{
    int i;
    char szCrossRef[CALLSSID_SIZE+1];
    size_t bytesread = 0;
    char szTemp[CALLSSID_SIZE+1];
    char szTarget[CALLSSID_SIZE+1];
    char *pTemp;

    strcpy( szTarget, szCall );
    FormatCall( szTarget );

    while ( bytesread < (size_t) m_Index.bytesperkey )
    {
        if ( fgets( m_szBuffer, QRZ_BUFFER_SIZE, m_hDbf) )
        {
            bytesread += strlen( m_szBuffer );
            m_pBuffer = m_szBuffer;
            pTemp = szTemp;
            for( i=0; i<6; i++ )
            {
                if( *m_pBuffer != ',' )
                    *pTemp++ = *m_pBuffer++;
            }
            *pTemp = 0;
            FormatCall( szTemp );
            if( memcmp( szTemp, szTarget, 6) == 0 )
            {
                if ( IsCrossRef( m_szBuffer ) )
                {
                    m_pBuffer++;
                    pTemp = strchr( m_pBuffer, '\n' );
                    if( pTemp )
                        *pTemp = 0;
                    ParseField( szCrossRef, CALLSSID_SIZE );    // the new call
                    if ( SetIndex ( szCrossRef ) )
                        return Search( szCrossRef );
                    else
                        return  FALSE;
                }
                return TRUE;
            }
        }
        else
            return FALSE;   // eof or error
    }

    return FALSE;
}



/////////////////////////////////////////////////////////////////////////////
// ParseField - parses out field from buffer string to pointer.  TRUE if
// success, FALSE if failure. Will accept NULL value for pDest. This allows
// skipping unwanted fields. Commas delimit the field. 'nMax' is the max
// number of characters that can be copied.
//

BOOL qrz::ParseField( char * pDest, int nMax )
{
    int nCnt;

    if( pDest )
        *pDest = 0;

    if ( *m_pBuffer == ',' )
    {
        m_pBuffer++;
        return FALSE;     // no field to parse
    }

    if ( *m_pBuffer ==  0 )
        return FALSE;

    nCnt = 0;
    FOREVER  //for(;;)
    {
        if ( ( *m_pBuffer == ',' ) || ( *m_pBuffer == 0 ) )
            break;
        if( pDest )
        {
            *pDest = *m_pBuffer;
            pDest++;
        }
        m_pBuffer++;
        nCnt++;
        if( nCnt >= nMax )
        {
            FOREVER // for(;;)
            {
                if( (*m_pBuffer == ',') || (*m_pBuffer == 0) )
                    break;
                m_pBuffer++;
            }
            break;
        }
    }

    if( pDest )
        *pDest = 0;

    m_pBuffer++;

    return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// IsCrossRef - checks record to see if it is a cross reference

BOOL qrz::IsCrossRef( char * szCall )
{
    char *pStr;
    int nCommas;

    pStr = szCall;
    nCommas = 0;

    while ( *pStr )
    {
        if ( *pStr == ',' )
        {
            nCommas++;
            if ( nCommas > 1 )
                return FALSE;
        }
        pStr++;
    }

    if ( nCommas == 0 )
        return FALSE;

    return TRUE;        // nCommas == 1
}

