/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// hconfig.cpp - wxnhostd.conf handler
//

#define _HCONFIG_CPP_

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

// libwxn
#include <datastruct.h>
#include <htoi.h>
#include <makeuc.h>
#include <cconfig.h>
#include <fips.h>
#include <logmsg.h>

#include "hconfig.h"
#include "precip.h"

// global

hostconf g_HostConfig;

// local

#define STNTYPE_SIZE 16

static char g_szStnType[STNTYPE_SIZE+1];


#define STRING  0
#define CHAR    1
#define BOOLEAN 2
#define INTEGER 3
#define SHORT   4
#define UNS_INT 5
#define TIME    6
#define LONG    7
#define MULTISZ 8
#define FLINT   9
#define FLOAT   10
#define SN_ARRAY   11

struct ini HostIni[] =
{
    { "City",                   STRING,     &g_HostConfig.szCity },
    { "County",                 STRING,     &g_HostConfig.szCounty },
    { "State",                  STRING,     &g_HostConfig.szState },
    { "StationType",            STRING,     &g_szStnType },
    { "AnemometerHeight",       INTEGER,    &g_HostConfig.nHAAT },
    { "Emp",                    BOOLEAN,    &g_HostConfig.bOwEmp },
    { "Rad",                    BOOLEAN,    &g_HostConfig.bOwRad },
    { "Cesium137",              INTEGER,    &g_HostConfig.nCesium137 },
    { "OwInsideTemp",           BOOLEAN,    &g_HostConfig.bOwInsideTemp },
    { "OwInsideTempSN",         SN_ARRAY,   &g_HostConfig.OwInsideTempSN },
    { "OwOutsideTemp",          BOOLEAN,    &g_HostConfig.bOwOutsideTemp },
    { "OwOutsideTempSN",        SN_ARRAY,   &g_HostConfig.OwOutsideTempSN },
    { "OwInsideHumidity",       BOOLEAN,    &g_HostConfig.bOwInsideHumid },
    { "OwInsideHumiditySN",     SN_ARRAY,   &g_HostConfig.OwInsideHumidSN },
    { "OwOutsideHumidity",      BOOLEAN,    &g_HostConfig.bOwOutsideHumid },
    { "OwOutsideHumiditySN",    SN_ARRAY,   &g_HostConfig.OwOutsideHumidSN },
    { "OwBarometer",            BOOLEAN,    &g_HostConfig.bOwBarom },
    { "OwBarometerSN",          SN_ARRAY,   &g_HostConfig.OwBaromSN },
    { "OwBarometerOffset",      INTEGER,    &g_HostConfig.nBaromOffset },
    { "OwSoilTemp",             BOOLEAN,    &g_HostConfig.bOwSoilTemp },
    { "OwSoilTempSN",           SN_ARRAY,   &g_HostConfig.OwSoilTempSN },
    { "OwSoil2Temp",            BOOLEAN,    &g_HostConfig.bOwSoil2Temp },
    { "OwSoil2TempSN",          SN_ARRAY,   &g_HostConfig.OwSoil2TempSN },
    { "Sunshine",               BOOLEAN,    &g_HostConfig.bOwSunshine },
    { "RainGaugeType",          INTEGER,    &g_HostConfig.nGaugeType },
    { "OwAspiratorCtrl",        BOOLEAN,    &g_HostConfig.bOwAspCtrl },
    { "OwAspiratorCtrlSN",      SN_ARRAY,   &g_HostConfig.OwAspCtrlSN },
    { "AspCtrlTrip",            INTEGER,    &g_HostConfig.nAspiratorTrip },
    { "OwHeaterCtrl",           BOOLEAN,    &g_HostConfig.bOwHtrCtrl },
    { "OwHeaterCtrlSN",         SN_ARRAY,   &g_HostConfig.OwHtrCtrlSN },
    { "WxPort",                 STRING,     &g_HostConfig.szWxStnPort },
    { "WxBaud",                 INTEGER,    &g_HostConfig.nBaud },
    { "OneWirePort",            STRING,     &g_HostConfig.szOneWirePort },
    { "NullNoSensors",          BOOLEAN,    &g_HostConfig.bNullNoSensors },
    { "WxPicPort",              STRING,     &g_HostConfig.szWxPicPort },
    { "OsBarometerOffset",      INTEGER,    &g_HostConfig.nBaromOffset },
    { "WxStnTimeout",           INTEGER,    &g_HostConfig.nTimeout },
    { "EmailAlertEnable",       BOOLEAN,    &g_HostConfig.email.bEnable },
    { "EmailHost",              STRING,     &g_HostConfig.email.sHost },
    { "EmailReplyTo",           STRING,     &g_HostConfig.email.sReplyTo },
    { "EmailFromAcct",          STRING,     &g_HostConfig.email.sFromAcct },
    { "EmailFromName",          STRING,     &g_HostConfig.email.sFromName }
};

#define HOST_INI_ENTRIES (sizeof(HostIni)/ sizeof(struct ini))


/////////////////////////////////////////////////////////////////////////////
// ReadHostConfig - read <wxnhostd.conf> to set required parameters.
// Required entries are city, county, and state.  Others optional.
// Returns TRUE success, FALSE failure.
//

BOOL ReadHostConfig( void )
{
    FILE * hFile;
    BOOL bFound;
    BOOL bNoMatch;
    int i;
    int n;
    char szString[STRING_SIZE];
    char szLine[STRING_SIZE];
    char szMsg[STRING_SIZE];
    char * pS;
    char * pParam;
    char * pVal;
    uchar * pSN;
    double dVal;

    hFile = fopen( WXN_SYSCONFDIR "wxnhostd.conf", "r" );
    if( hFile == NULL )
    {
        g_log.PostMsg( HOSTMSG_WXNHOSTD_CONF_FAIL, "cannot find/open file" );
        return FALSE;
    }

    // set defaults

    *g_szStnType = 0;

    memset( &g_HostConfig, 0, sizeof(struct hostconf) );

    g_HostConfig.nGaugeType = HUNDREDTHS;
    // Alarm defaults set by ReadAlarmConfig()
    g_HostConfig.nCesium137 = 1050;
    g_HostConfig.bNullNoSensors = TRUE;
    g_HostConfig.nAspiratorTrip = 80;

    FOREVER     // for(;;)
    {
        if( fgets( szLine, STRING_SIZE, hFile ) == NULL )
            break;      // EOF

        if( *szLine == '\r' || *szLine == '\n' )
            continue;               // empty line

        pS =strchr( szLine, 0 );
        if( *(pS-1) == '\r' || *(pS-1) == '\n' )
            *(pS-1) = 0;

        strcpy( szString, szLine );

        bFound = FALSE;
        bNoMatch = TRUE;

        pParam = szString;
        while( *pParam )
        {
            if( *pParam != ' ' )
                break;
            pParam++;
        }

        if( *pParam == '#' )      // comment
            continue;

        pVal = pParam;
        while( *pVal )
        {
            if( *pVal == ' ' ) {
                *pVal++ = 0;        // terminate the parameter portion of string
                continue;
                }
            if( *pVal++ == '=' )
                break;
        }
        while( *pVal )
        {
            if( *pVal != ' ' )
                break;
            pVal++;
        }

        if( *pVal == '"' )          // check for enclosing quotes
            pVal++;
        if( strchr( pVal, '"' ) )
            *strchr( pVal, '"' ) = 0;

        if( *pVal == 0 )
        {
            sprintf( szMsg, "don't understand <%s>", szLine );
            g_log.PostMsg( HOSTMSG_WXNHOSTD_CONF_FAIL, szMsg );
            continue;
        }

        for( i=0; i<(int)HOST_INI_ENTRIES; i++ )
        {
            if( strcasecmp( pParam, HostIni[i].szParam ) == 0 )
            {
                bFound = TRUE;
                bNoMatch = FALSE;
                if( pVal == NULL )
                    break;
                switch( HostIni[i].nType )
                {
                    case STRING:
                        strcpy( (char *) HostIni[i].pType, pVal );
                        break;

                    case CHAR:
                        *(char *) HostIni[i].pType = *pVal;
                        break;

                    case BOOLEAN:
                        if( ((*pVal & 0x005f) == 'T') || ((*pVal & 0x005f) == 'Y') )
                            *((BOOL *) HostIni[i].pType) = TRUE;
                        else
                            *((BOOL *) HostIni[i].pType) = FALSE;
                        break;

                    case INTEGER:
                        *((int *) HostIni[i].pType) = atoi( pVal );
                        break;

                    case SHORT:
                        *((short *) HostIni[i].pType) = (short) atoi( pVal );
                        break;

                    case UNS_INT:
                        *((unsigned *) HostIni[i].pType) = (unsigned) atol( pVal );
                        break;

                    case TIME:
                        *((time_t *) HostIni[i].pType) = (time_t) atol( pVal );
                        break;

                    case LONG:
                        *((long *) HostIni[i].pType) = atol( pVal );
                        break;

                    case FLINT:
                        dVal = atof( pVal ) * 100;
                        *((int *) HostIni[i].pType) = (int) dVal;   // one-liner introduces error
                        break;

                    case FLOAT:
                        dVal = atof( pVal );
                        *((double *) HostIni[i].pType) = dVal;
                        break;

                    case SN_ARRAY:
                        pSN = (uchar *) HostIni[i].pType;
                        for( n=0; n<OW_SN_LEN; n++ )
                            *(pSN+((OW_SN_LEN-1)-n)) = (uchar) htoi( pVal+(n<<1), 2 );
                        break;

                    default:
                        bNoMatch = TRUE;
                        break;
                }
            }

            if( bFound )
                break;
        }

        if( bNoMatch )
        {
            sprintf( szMsg, "don't understand <%s>", szLine );
            g_log.PostMsg( HOSTMSG_WXNHOSTD_CONF_FAIL, szMsg );
        }
    }

    fclose( hFile );

    makeuc( g_ComConfig.szAlias );
    makeuc( g_HostConfig.szState );

    if( *g_szStnType == 0 )
    {
        g_log.PostMsg( HOSTMSG_WXNHOSTD_CONF_FAIL, "no <StationType> defined for wx station" );
        return FALSE;
    }
    else
    {
        if( strcasecmp( g_szStnType, "ID5" ) == 0 )
            g_HostConfig.nStnType = ID5001;
        else if( strcasecmp( g_szStnType, "UM2" ) == 0 )
            g_HostConfig.nStnType = ULTII;
        else if( strcasecmp( g_szStnType, "U2K" ) == 0 )
            g_HostConfig.nStnType = ULT2000;
        else if( strcasecmp( g_szStnType, "T238" ) == 0 )
            g_HostConfig.nStnType = ULT2000;
        else if( strcasecmp( g_szStnType, "DAV2" ) == 0 )
            g_HostConfig.nStnType = DAVIS2;
        else if( strcasecmp( g_szStnType, "DAVP" ) == 0 )
            g_HostConfig.nStnType = DAVISVP;
        else
        {
            g_log.PostMsg( HOSTMSG_WXNHOSTD_CONF_FAIL, "no valid <StationType> for wx station" );
            return FALSE;
        }
    }

    if( g_HostConfig.nTimeout == 0 )    // no entry in config file
    {
        // may handle this differently as more drivers are added
        switch( g_HostConfig.nStnType )
        {
            case DAVIS2:
            case ID5001:
            case DAVISVP:
                g_HostConfig.nTimeout = 5;
                break;

            default:
                g_HostConfig.nTimeout = 15;
                break;
        }
    }

    if( *g_HostConfig.szWxStnPort == 0 )
    {
        g_log.PostMsg( HOSTMSG_WXNHOSTD_CONF_FAIL, "no <WxPort> defined for wx station" );
        return FALSE;
    }

    if( g_HostConfig.bOwEmp || g_HostConfig.bOwRad || g_HostConfig.bOwBarom || g_HostConfig.bOwInsideTemp ||
        g_HostConfig.bOwOutsideTemp || g_HostConfig.bOwInsideHumid || g_HostConfig.bOwOutsideHumid ||
        g_HostConfig.bOwSoilTemp )
    {
        if( *g_HostConfig.szOneWirePort == 0 )
        {
            g_log.PostMsg( HOSTMSG_WXNHOSTD_CONF_FAIL, "no <OneWirePort> defined" );
            return FALSE;
        }
        g_HostConfig.bEnableOneWire = TRUE;
    }

    if( (*g_HostConfig.szCity == 0) && (*g_HostConfig.szCounty == 0) && (*g_HostConfig.szState == 0) )
    {
        g_log.PostMsg( HOSTMSG_WXNHOSTD_CONF_FAIL, "city, county, state not defined" );
        return FALSE;
    }

    if( !SetFips( g_HostConfig.szState, g_HostConfig.szCounty, g_HostConfig.szCity, g_HostConfig.szFips ) )
    {
        sprintf( szMsg, "no matching entry for: %s, %s (%s) in geographic database",
            g_HostConfig.szCity, g_HostConfig.szState, g_HostConfig.szCounty );
        g_log.PostMsg( HOSTMSG_WXNHOSTD_CONF_FAIL, szMsg );
        return FALSE;
    }

    if( g_HostConfig.nBaud == 0 )
    {
        switch( g_HostConfig.nStnType )
        {
            case ULT2000:
            case ULTII:
                g_HostConfig.nBaud = 2400;
                break;

            case ID5001:
                g_HostConfig.nBaud = 9600;
                break;

            case DAVIS2:
                g_HostConfig.nBaud = 2400;
                break;

            case DAVISVP:
                g_HostConfig.nBaud = 19200;
                break;
        }

    }

    if( *g_HostConfig.szWxPicPort )
        g_HostConfig.bEnableWxPic = TRUE;

    if( g_HostConfig.email.bEnable )
    {
        if( (*g_HostConfig.email.sHost == 0) || (*g_HostConfig.email.sReplyTo == 0) || (*g_HostConfig.email.sFromAcct == 0) )
        {
            g_HostConfig.email.bEnable = FALSE;
            g_log.PostMsg( HOSTMSG_WXNHOSTD_CONF_FAIL, "Email alerts disabled. Email account name, host name "
                "and reply-to account must be furnished." );
        }
    }

    return TRUE;
}

