/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// qthinfo.cpp - set local information for wx node
//

#define _QTHINFO_CPP_

#include <stdlib.h>
#include <string.h>

#include <libpq-fe.h>

// libwxn
#include <strxcpy.h>
#include <cconfig.h>

#include "hconfig.h"
#include "wxdata.h"
#include "qthinfo.h"


BOOL SetLocalInfo( void )
{
    char szExec[384];
    BOOL bStatus;
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( PQstatus( pConn ) != CONNECTION_OK )
        return FALSE;

    // always overwrite <g_Local.Qth> to insure it is up to date
    strxcpy( g_Local.Info.szCallSsid, g_ComConfig.szServerCall, CALLSSID_SIZE );
    strxcpy( g_Local.Info.szAlias, g_ComConfig.szAlias, ALIAS_SIZE );
    g_Local.Info.nHAAT = g_HostConfig.nHAAT;
    strxcpy( g_Local.Info.szCity, g_HostConfig.szCity, CITY_SIZE );
    strxcpy( g_Local.Info.szCounty, g_HostConfig.szCounty, COUNTY_SIZE );
    strxcpy( g_Local.Info.szState, g_HostConfig.szState, STATE_SIZE );
    g_Local.Info.nAprSoftType = 1;
    g_Local.Info.nStnType = g_HostConfig.nStnType;
    g_Local.Info.dLatitude = g_ComConfig.dLatitude;
    g_Local.Info.dLongitude = g_ComConfig.dLongitude;

    sprintf( szExec, "SELECT afos, fips, zone, elev, dst "
        "FROM gdb WHERE state='%s' AND county='%s' AND city='%s'",
        g_Local.Info.szState, g_Local.Info.szCounty, g_Local.Info.szCity );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
    {
        strcpy( g_Local.Info.szAfos, PQgetvalue( pResult, 0, 0 ) );
        strcpy( g_Local.Info.szFips, PQgetvalue( pResult, 0, 1 ) );
        g_Local.Info.nTimeZone = atoi( PQgetvalue( pResult, 0, 2 ) );

        if( PQgetisnull( pResult, 0, 3 ) )
            g_Local.Info.nElevation = 0;
        else
            g_Local.Info.nElevation = atoi( PQgetvalue( pResult, 0, 3 ) );

        if(  *PQgetvalue( pResult, 0, 4 ) == 't' )
            g_Local.Info.bDst = TRUE;
        else
            g_Local.Info.bDst = FALSE;
    }
    else
    {
        PQclear( pResult );
        PQfinish( pConn );
        return FALSE;
    }
    PQclear( pResult );

    // check for existence and create record if necessary
    sprintf( szExec, "SELECT call_ssid FROM nodes WHERE call_ssid='%s'", g_ComConfig.szServerCall );
    pResult = PQexec( pConn, szExec );
    if( PQresultStatus( pResult ) != PGRES_TUPLES_OK )
    {
        PQclear( pResult );
        PQfinish( pConn );
        return FALSE;
    }
    if( PQntuples( pResult ) == 0 )
    {
        // record does not exist
        PQclear( pResult );
        sprintf( szExec, "INSERT INTO nodes( call_ssid ) VALUES( '%s' )", g_ComConfig.szServerCall );
        pResult = PQexec( pConn, szExec );
        if( PQresultStatus( pResult ) != PGRES_COMMAND_OK )
        {
            PQclear( pResult );
            PQfinish( pConn );
            return FALSE;
        }
    }
    PQclear( pResult );

    sprintf( szExec, "UPDATE nodes SET city='%s', county='%s', state='%s', haat=%u, alias='%s', "
        "longitude=%f, latitude=%f, timezone=%i, elevation=%i, dst='%c', afos='%s', aprs_soft=%i, "
        "stn_type=%i, fips='%s' WHERE call_ssid='%s'",
        g_HostConfig.szCity, g_HostConfig.szCounty, g_HostConfig.szState, g_HostConfig.nHAAT, g_ComConfig.szAlias,
        g_Local.Info.dLongitude, g_Local.Info.dLatitude, g_Local.Info.nTimeZone, g_Local.Info.nElevation,
        g_Local.Info.bDst ? 't' : 'f', g_Local.Info.szAfos, g_Local.Info.nAprSoftType, g_Local.Info.nStnType,
        g_Local.Info.szFips, g_ComConfig.szServerCall );
    pResult = PQexec( pConn, szExec );
    bStatus = TRUE;
    if( PQresultStatus( pResult ) != PGRES_COMMAND_OK )
        bStatus = FALSE;

    PQclear( pResult );

    PQfinish( pConn );

    return bStatus;
}

