/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define _RATES_CPP_

#include <stdlib.h>
#include <libpq-fe.h>

#include "hglobal.h"
#include "wxdata.h"
#include "sensors.h"
#include "rates.h"


//////////////////////////////////////////////////////////////////////////////
// RatesInit - if called with 'bForce' == TRUE, then force initialization of
// rate tables to default values (current or zero, depending on type). If
// current value of rate table for given parm is -999, table is initialized
// to default. If current value != -999 and 'bForce' == FALSE, no action is
// taken (assumed to be restart and values in table are legit).
//

void RatesInit( BOOL bForce )
{
    int nVal;
    BOOL bInit;
    PGconn * pConn;
    PGresult * pResult;
    char szExec[128];

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn )
    {
        if( PQstatus( pConn ) == CONNECTION_OK )
        {
            pResult = PQexec( pConn, "BEGIN work" );
            PQclear( pResult );

            ////////////////////////// temp
            nVal = g_Local.Data.tcr;    // default
            bInit = FALSE;
            if( !bForce )
            {
                pResult = PQexec( pConn, "SELECT tcr FROM temp WHERE hour=0 AND min=0" );
                if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
                {
                    if( PQntuples( pResult ) )
                    {
                        if( atoi( PQgetvalue( pResult, 0, 0 ) ) == -999 )
                            bInit = TRUE;
                    }
                }
                PQclear( pResult );
            }

            if( bForce || bInit )
            {
                sprintf( szExec, "UPDATE temp SET tcr=%i", nVal );
                pResult = PQexec( pConn, szExec );
                PQclear( pResult );
            }

            ////////////////////////// precip
            bInit = FALSE;
            if( !bForce )
            {
                pResult = PQexec( pConn, "SELECT plt FROM precip WHERE hour=0 AND min=0" );
                if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
                {
                    if( PQntuples( pResult ) )
                    {
                        if( atoi( PQgetvalue( pResult, 0, 0 ) ) == -999 )
                            bInit = TRUE;
                    }
                }
                PQclear( pResult );
            }

            if( bForce || bInit )
            {
                sprintf( szExec, "UPDATE precip SET plt=0" );
                pResult = PQexec( pConn, szExec );
                PQclear( pResult );
            }

            ////////////////////////// barom
            if( g_Sensors.bBarom || g_Sensors.bOwBarom )
            {
                nVal = g_Local.Data.bcr;    // default
                bInit = FALSE;
                if( !bForce )
                {
                    pResult = PQexec( pConn, "SELECT bcr FROM barom WHERE hour=0 AND min=0" );
                    if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
                    {
                        if( PQntuples( pResult ) )
                        {
                            if( atoi( PQgetvalue( pResult, 0, 0 ) ) == -999 )
                                bInit = TRUE;
                        }
                    }
                    PQclear( pResult );
                }

                if( bForce || bInit )
                {
                    sprintf( szExec, "UPDATE barom SET bcr=%i", nVal );
                    pResult = PQexec( pConn, szExec );
                    PQclear( pResult );
                }
            }

            ////////////////////////// humid
            if( g_Sensors.bOutsideHumid || g_Sensors.bOwOutsideHumid )
            {
                nVal = g_Local.Data.hcr;    // default
                bInit = FALSE;
                if( !bForce )
                {
                    pResult = PQexec( pConn, "SELECT hcr FROM humid WHERE hour=0 AND min=0" );
                    if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
                    {
                        if( PQntuples( pResult ) )
                        {
                            if( atoi( PQgetvalue( pResult, 0, 0 ) ) == -999 )
                                bInit = TRUE;
                        }
                    }
                    PQclear( pResult );
                }

                if( bForce || bInit )
                {
                    sprintf( szExec, "UPDATE humid SET hcr=%i", nVal );
                    pResult = PQexec( pConn, szExec );
                    PQclear( pResult );
                }
            }

            ////////////////////////// rad
            if( g_Sensors.bOwRad )
            {
                nVal = 0;    // default
                bInit = FALSE;
                if( !bForce )
                {
                    pResult = PQexec( pConn, "SELECT rlt FROM rad WHERE hour=0 AND min=0" );
                    if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
                    {
                        if( PQntuples( pResult ) )
                        {
                            if( atoi( PQgetvalue( pResult, 0, 0 ) ) == -999 )
                                bInit = TRUE;
                        }
                    }
                    PQclear( pResult );
                }

                if( bForce || bInit )
                {
                    sprintf( szExec, "UPDATE rad SET rlt=NULL" );
                    pResult = PQexec( pConn, szExec );
                    PQclear( pResult );
                }
            }
        }

        pResult = PQexec( pConn, "COMMIT work" );
        PQclear( pResult );

        PQfinish( pConn );
    }
}

