/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// temp.cpp - data handlers for temperature
//

#define _TEMP_CPP_

#include <stdlib.h>
#include <math.h>
#include <libpq-fe.h>

// libwxn
#include <tz.h>

#include "wxdata.h"
#include "temp.h"


//////////////////////////////////////////////////////////////////////////////
// TempSet = sets tcr, thr, tlr, trh, trd
//

void TempSet( int nTemp, int nHigh, int nLow )
{
    int nMin;
    PGconn * pConn;
    PGresult * pResult;
    char szExec[128];

    // current
    g_Local.Data.tcr = nTemp;

    //// high temp
    if( g_Local.Data.thr.val <= nHigh )
    {
        g_Local.Data.thr.val = nHigh;
        g_Local.Data.thr.dtg = g_Time.Qth();
    }

    //// low temp
    if( g_Local.Data.tlr.val >= nLow )
    {
        g_Local.Data.tlr.val = nLow;
        g_Local.Data.tlr.dtg = g_Time.Qth();
    }

    // hourly/24 hour rate
    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn )
    {
        if( PQstatus( pConn ) == CONNECTION_OK )
        {
            // force minute to be multiple of 5
            nMin = (g_Time.Minute()/5) * 5;

            // hourly rate
            sprintf( szExec, "SELECT tcr FROM temp WHERE hour=%i AND min=%i", (g_Time.Hour()+23) % 24, nMin );
            pResult = PQexec( pConn, szExec );
            if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
            {
                if( PQntuples( pResult ) )
                    g_Local.Data.trh = nTemp - atoi( PQgetvalue( pResult, 0, 0 ) );
                else
                    g_Local.Data.trh = 0;
            }
            PQclear( pResult );

            // 24 hour rate
            sprintf( szExec, "SELECT tcr FROM temp WHERE hour=%i AND min=%i", g_Time.Hour(), nMin );
            pResult = PQexec( pConn, szExec );
            if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
            {
                if( PQntuples( pResult ) )
                    g_Local.Data.trd = nTemp - atoi( PQgetvalue( pResult, 0, 0 ) );
                else
                    g_Local.Data.trd = 0;
            }

            // save current reading - must come ***AFTER*** daily rate is determined.
            sprintf( szExec, "UPDATE temp SET tcr=%i WHERE hour=%i AND min=%i",
                nTemp, g_Time.Hour(), nMin );
            pResult = PQexec( pConn, szExec );
            PQclear( pResult );

        }

        PQfinish( pConn );

    }
}


//////////////////////////////////////////////////////////////////////////////
// TempInitSet
//

void TempInitSet( int nTemp )
{
    g_Local.Data.tcr = nTemp;
}


//////////////////////////////////////////////////////////////////////////////
// InsideTempSet = sets inside temperature
//

void InsideTempSet( int nTemp )
{
    g_Local.Data.ist = nTemp;
}

