//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Dallas Semiconductor
// shall not be used except as stated in the Dallas Semiconductor
// Branding Policy.
// ---------------------------------------------------------------------------
//
//  temp10.C - Module to read the DS1920/DS1820 - temperature measurement.
//
//  Version: 2.00
//
// ---------------------------------------------------------------------------
//
//

#include <string.h>

#include "ownet.h"
#include "temp10.h"


//----------------------------------------------------------------------
// Read the temperature of a DS1920/DS1820
//
// 'SerialNum'   - Serial Number of DS1920/DS1820 to read temperature from
// 'Temp '       - pointer to variable where that temperature will be
//                 returned
//
// Returns: TRUE(1)  temperature has been read and verified
//          FALSE(0) could not read the temperature, perhaps device is not
//                   in contact
//

int DS1820Temperature( uchar * pSerialNum, int * pTemp )
{

    uchar data_block[30];
    uchar lastcrc8;
    int send_cnt;
    int i;
    int loop;
    short nTemp;
    BOOL bRoundUp;

    // set the device serial number to the temp device
    owSerialNum( pSerialNum, FALSE );

    for( loop=0; loop<3; loop ++ )
    {
        send_cnt = 0;
        setcrc8( 0 );

        // access the device
        if ( owAccess() )
        {
            // send the convert command and start power delivery
            if( !owWriteBytePower( 0x44 ) )
                return FALSE;

            // sleep for 1 second
            msDelay( 1000 );

            // turn off the 1-Wire Net strong pull-up
            if( owLevel(MODE_NORMAL) != MODE_NORMAL )
                return FALSE;

            // access the device
            if ( owAccess() )
            {
                // create a block to send that reads the temperature
                // read scratchpad command
                data_block[send_cnt++] = 0xBE;

                // now add the read bytes for data bytes and crc8
                for (i = 0; i < 9; i++)
                    data_block[send_cnt++] = 0xFF;

                // now send the block
                if (owBlock(FALSE, data_block, send_cnt) )
                {
                    // perform the CRC8 on the last 8 bytes of packet
                    lastcrc8 = 0xff;    // just in case...
                    for (i = send_cnt - 9; i < send_cnt; i++)
                        lastcrc8 = docrc8(data_block[i]);

                    // verify CRC8 is correct
                    if( lastcrc8 == 0x00 )
                    {
                        nTemp = data_block[1] | (data_block[2] << 8);
                        nTemp *= 90;
                        if( (nTemp % 100) < 50 )
                            bRoundUp = FALSE;
                        else
                            bRoundUp = TRUE;
                        *pTemp = (int) nTemp/100 + 32;    // correct since binary is actually 2x C.
                        if( bRoundUp )
                        (*pTemp)++;

                        // success
                        return TRUE;
                    }
                    else
                        OWERROR( OWERROR_CRC_FAILED );
                }
            }
        }
    }

   return FALSE;

}

